@@ -10,8 +10,7 @@ import StringIO, os, re, random, codecs
 
 from MoinMoin import config, caching, user, util, wikiutil
 from MoinMoin.logfile import eventlog
-from MoinMoin.util import filesys
-from MoinMoin.util.datetime import formathttpdate
+from MoinMoin.util import filesys, MoinMoinNoFooter, timefuncs
 
 class Page:
     """Page - Manage an (immutable) page associated with a WikiName.
@@ -606,7 +605,7 @@ class Page:
         If you want to get a more accurate number, call with
         exists=1. This will be about 100 times slower though.
 
-        @param exits: filter existing pages
+        @param exists: filter existing pages
         @rtype: int
         @return: number of pages
         """
@@ -624,7 +623,7 @@ class Page:
         return count
 
     def getPageList(self, user=None, exists=1, filter=None):
-        ''' List user readable pages under current page
+        """ List user readable pages under current page
 
         Currently only request.rootpage is used to list pages, but if we
         have true sub pages, any page can list its sub pages.
@@ -648,10 +647,10 @@ class Page:
 
         @param user: the user requesting the pages (MoinMoin.user.User)
         @param filter: filter function
-        @param exits: filter existing pages
+        @param exists: filter existing pages
         @rtype: list of unicode strings
         @return: user readable wiki page names
-        '''
+        """
         request = self.request
         request.clock.start('getPageList')
         # Check input
@@ -703,7 +702,7 @@ class Page:
 
         @param user: the user requesting the pages
         @param filter: filter function
-        @param exits: only existing pages
+        @param exists: only existing pages
         @rtype: dict {unicode: Page}
         @return: user readable pages
         """
@@ -761,17 +760,17 @@ class Page:
         return pages
 
     def getlines(self):
-        """Return a list of all lines in body.
+        """ Return a list of all lines in body.
 
-        @rtype list
+        @rtype: list
         @return: list of strs body_lines"""
         lines = self.get_raw_body().split('\n')
         return lines
 
     def get_raw_body(self):
-        """Load the raw markup from the page file.
+        """ Load the raw markup from the page file.
 
-        @rtype str
+        @rtype: str
         @return: raw page contents of this page
         """
         if self._raw_body is None:
@@ -817,7 +816,7 @@ class Page:
         @param request: the request object
         @param querystr: the query string to add after a "?" after the url
             (str or dict, see wikiutil.makeQueryString)
-        @param escpae: escape url for html, to be backward compatible
+        @param escape: escape url for html, to be backward compatible
             with old code (bool)
         @rtype: str
         @return: complete url of this page, including scriptname
@@ -934,7 +933,8 @@ class Page:
 
             if subscriber.isSubscribedTo(pageList):
                 lang = subscriber.language or 'en'
-                if not subscriber_list.has_key(lang): subscriber_list[lang] = []
+                if not lang in subscriber_list:
+                    subscriber_list[lang] = []
                 if return_users:
                     subscriber_list[lang].append(subscriber)
                 else:
@@ -943,6 +943,15 @@ class Page:
         return subscriber_list
 
 
+    def send_raw(self):
+        """ Output the raw page data (action=raw) """
+        self.request.http_headers(["Content-type: text/plain;charset=%s" % config.charset])
+        text = self.get_raw_body()
+        text = self.encodeTextMimeType(text)
+        self.request.write(text)
+        raise MoinMoinNoFooter
+
+
     def send_page(self, request, msg=None, **keywords):
         """ Output the formatted page.
 
@@ -1129,7 +1138,7 @@ class Page:
                     # use the correct last-modified value from the on-disk file
                     # to ensure cacheability where supported
                     request.http_headers(["Last-Modified: " +
-                         formathttpdate(os.path.getmtime(self._text_filename()))])
+                         timefuncs.formathttpdate(os.path.getmtime(self._text_filename()))])
 
             else:
                 request.http_headers(['Status: 404 NOTFOUND'])
