@@ -13,10 +13,9 @@ from MoinMoin.Page import Page
 from MoinMoin.widget import html
 from MoinMoin.widget.dialog import Status
 from MoinMoin.logfile import editlog, eventlog
-from MoinMoin.util import filesys
+from MoinMoin.util import filesys, timefuncs
 import MoinMoin.util.web
 import MoinMoin.util.mail
-import MoinMoin.util.datetime
 
 
 #############################################################################
@@ -112,6 +111,19 @@ class PageEditor(Page):
 
         return False
 
+    def sendconfirmleaving(self):
+        """ to prevent moving away from the page without saving it """
+        _ = self._
+        self.request.write(u'''\
+<script type="text/javascript">
+    var flgChange = false;
+    function confirmleaving() {
+        if ( flgChange )
+            return "%s";
+    }
+</script>
+''' % _("Your changes are not saved!"))
+        
     def sendEditor(self, **kw):
         """
         Send the editor form page.
@@ -314,15 +326,22 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
                      'license_link': wikiutil.getSysPage(self.request, self.cfg.page_license_page).link_to(self.request),
             }, '</em></p>')
 
+
         self.request.write('''
 <input class="button" type="submit" name="button_save" value="%s" onClick="flgChange = false;">
 <input class="button" type="submit" name="button_preview" value="%s" onClick="flgChange = false;">
-<input id="switch2gui" style="display: none;" class="button" type="submit"     
-    name="button_switch" value="%s">
+''' % (save_button_text, _('Preview'),))
+        
+        if not (self.request.cfg.editor_force and self.request.cfg.editor_default == 'text'):
+            self.request.write('''
+<input id="switch2gui" style="display: none;" class="button" type="submit" name="button_switch" value="%s">
+''' % (_('GUI Mode'),))
+            
+        self.request.write('''
 %s
 <input class="button" type="submit" name="button_cancel" value="%s">
 <input type="hidden" name="editor" value="text">
-''' % (save_button_text, _('Preview'), _('GUI Mode'), button_spellcheck, cancel_button_text,))
+''' % (button_spellcheck, cancel_button_text,))
 
         # Add textarea with page text
 
@@ -331,16 +350,7 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
         # language into meta file.
         lang = self.language or self.request.cfg.language_default
 
-        # to prevent moving away from the page without saving it
-        self.request.write(u'''\
-<script type="text/javascript">
-    var flgChange = false;
-    function confirmleaving() {
-        if ( flgChange )
-            return "%s";
-    }
-</script>
-''' % _("Your changes are not saved!"))
+        self.sendconfirmleaving()
 
         self.request.write(
             u'''\
@@ -402,16 +412,12 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
         self.request.write("</form>")
         
         # QuickHelp originally by Georg Mischler <schorsch@lightingwiki.com>
-        self.request.write(self.request.formatter.div(1, id="editor-help"))
-        self.request.write(_(""" Emphasis:: [[Verbatim('')]]''italics''[[Verbatim('')]]; [[Verbatim(''')]]'''bold'''[[Verbatim(''')]]; [[Verbatim(''''')]]'''''bold italics'''''[[Verbatim(''''')]]; [[Verbatim('')]]''mixed ''[[Verbatim(''')]]'''''bold'''[[Verbatim(''')]] and italics''[[Verbatim('')]]; [[Verbatim(----)]] horizontal rule.
- Headings:: [[Verbatim(=)]] Title 1 [[Verbatim(=)]]; [[Verbatim(==)]] Title 2 [[Verbatim(==)]]; [[Verbatim(===)]] Title 3 [[Verbatim(===)]];   [[Verbatim(====)]] Title 4 [[Verbatim(====)]]; [[Verbatim(=====)]] Title 5 [[Verbatim(=====)]].
- Lists:: space and one of: * bullets; 1., a., A., i., I. numbered items; 1.#n start numbering at n; space alone indents.
- Links:: [[Verbatim(JoinCapitalizedWords)]]; [[Verbatim(["brackets and double quotes"])]]; url; [url]; [url label].
- Tables:: || cell text |||| cell text spanning 2 columns ||;    no trailing white space allowed after tables or titles.
-
-(!) For more help, see HelpOnEditing or SyntaxReference.
-"""))
-        self.request.write(self.request.formatter.div(0))
+        markup = self.pi_format or self.request.cfg.default_markup
+        quickhelp = self.request.cfg.editor_quickhelp.get(markup, "")
+        if quickhelp:
+            self.request.write(self.request.formatter.div(1, id="editor-help"))
+            self.request.write(_(quickhelp))
+            self.request.write(self.request.formatter.div(0))
 
         if preview is not None:
             if staytop:
@@ -583,7 +589,7 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
         """
         # TODO: Allow addition of variables via wikiconfig or a global
         # wiki dict.
-        now = time.strftime("%Y-%m-%dT%H:%M:%SZ", util.datetime.tmtuple())
+        now = time.strftime("%Y-%m-%dT%H:%M:%SZ", timefuncs.tmtuple())
         user = self.request.user
         signature = user.signature()
         variables = {
@@ -647,7 +653,7 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
         Replace text of the user UserName/MoinEditorBackup with newtext.
 
         @param newtext: new text of the page
-        @keyword: none used currently
+        @param kw: no keyword args used currently
         @rtype: unicode
         @return: url of page backup or None
         """
