@@ -17,7 +17,6 @@ from MoinMoin.logfile import editlog, eventlog
 from MoinMoin.util import filesys
 import MoinMoin.util.web
 import MoinMoin.util.mail
-import MoinMoin.util.datetime
 from MoinMoin.parser.wiki import Parser
 
 from StringIO import StringIO
@@ -243,6 +242,8 @@ If you don't want that, hit '''%(cancel_button_text)s''' to cancel your changes.
 <input type="hidden" name="editor" value="gui">
 ''' % (save_button_text, _('Preview'), _('Text mode'), button_spellcheck, cancel_button_text,))
 
+        self.sendconfirmleaving() # TODO update state of flgChange to make this work, see PageEditor
+
         # Add textarea with page text
 
         # TODO: currently self.language is None at this point. We have
