@@ -1,10 +1,10 @@
 # -*- coding: iso-8859-1 -*-
 """
-    MoinMoin
+    MoinMoin - <short description>
 
     <what this stuff does ... - verbose enough>
 
-    @copyright: 2004 by Jrgen Hermann <jh@web.de>
+    @copyright: 2006 by MoinMoin:YourNameHere 
     @license: GNU GPL, see COPYING for details.
 """
 
