@@ -8,7 +8,7 @@
     @license: GNU GPL, see COPYING for details.
 """
 
-import unittest
+import unittest 
 from MoinMoin._tests import TestConfig
 from MoinMoin import config
 
@@ -121,3 +121,37 @@ class HTTPDateTests(unittest.TestCase):
         self.failUnlessEqual(self.request.httpDate(0, rfc='850'), 
                              'Thursday, 01-Jan-70 00:00:00 GMT',
                              'wrong date string')
+                             
+
+class GetPageNameFromQueryString(unittest.TestCase):
+    """ Test urls like http://netloc/wiki?pagename """
+
+    def setUp(self):
+        self.savedQuery = self.request.query_string
+    
+    def tearDown(self):
+        self.request.query_string = self.savedQuery
+    
+    def testAscii(self):
+        """ request: getPageNameFromQueryString: ascii """
+        name = expected = u'page name'
+        self.runTest(name, expected)
+        
+    def testNonAscii(self):
+        """ request: getPageNameFromQueryString: non ascii """
+        name = expected = u'דף עברי'
+        self.runTest(name, expected)
+
+    def testUnderscore(self):
+        """ request: getPageNameFromQueryString: under_score """
+        name = u'page_name'
+        expected = u'page name'
+        self.runTest(name, expected)
+        
+    def runTest(self, name, expected):
+        import urllib
+        # query as made by most browsers when you type the url into the
+        # location box.
+        query = urllib.quote(name.encode('utf-8'))
+        self.request.query_string = query
+        self.assertEqual(self.request.getPageNameFromQueryString(), expected)
