@@ -7,7 +7,7 @@
 """
 
 import unittest, os
-from MoinMoin import _tests
+
 from MoinMoin import wikimacro, wikiutil
 from MoinMoin.parser.plain import Parser
 from MoinMoin.formatter.text_html import Formatter
@@ -16,25 +16,17 @@ from MoinMoin.formatter.text_html import Formatter
 class MacroTestCase(unittest.TestCase):
     def testTrivialMacro(self):
         """wikimacro: trivial macro works"""
-        m = _make_macro()
+        m = self._make_macro()
         expected = m.formatter.linebreak(0)
         result = m.execute("BR", "")
         self.assertEqual(result, expected,
-            'Expected "%(expected)s" but got "%(result)s"' % locals())
-        
+            'Expected "%(expected)s" but got "%(result)s"' % locals())        
 
-def execute(macro, args):
-    """Test helper"""
-    assert hasattr(macro, "request")
-    assert hasattr(macro.request, "user")
-    return args
-
-
-def _make_macro():
-    """Test helper"""
-    p = Parser("##\n", _tests.request)
-    p.formatter = Formatter(_tests.request)
-    _tests.request.formatter = p.formatter
-    p.form = _tests.request.form
-    m = wikimacro.Macro(p)
-    return m
+    def _make_macro(self):
+        """Test helper"""
+        p = Parser("##\n", self.request)
+        p.formatter = Formatter(self.request)
+        self.request.formatter = p.formatter
+        p.form = self.request.form
+        m = wikimacro.Macro(p)
+        return m
