@@ -25,7 +25,6 @@
     
 from MoinMoin import wikiutil
 from MoinMoin.Page import Page
-from MoinMoin.formatter.text_html import Formatter
 
 def execute(pagename, request):
     _ = request.getText
@@ -39,13 +38,12 @@ def execute(pagename, request):
 
     # Start content - IMPORTANT - witout content div, there is no
     # direction support!
-    formatter = Formatter(request)
-    request.write(formatter.startContent("content"))
+    request.write(request.formatter.startContent("content"))
 
     request.write(LocalSiteMap(pagename).output(request))
 
     # End content
-    request.write(formatter.endContent()) # end content div
+    request.write(request.formatter.endContent()) # end content div
     # Footer
     wikiutil.send_footer(request, pagename)
 
