@@ -66,12 +66,14 @@ def execute(pagename, request, fieldname='value', titlesearch=0):
     # XXX won't work with attachment search
     # improve if we have one...
     if len(results.hits) == 1:
-        page = Page(request, results.hits[0].page_name)
-        # TODO: remove escape=0 in 2.0
-        url = page.url(request, querystr={'highlight': query.highlight_re()},
-                       escape=0)
-        request.http_redirect(url)
-        raise MoinMoinNoFooter
+        page = results.hits[0]
+        if not page.attachment: # we did not find an attachment
+            page = Page(request, page.page_name)
+            # TODO: remove escape=0 in 2.0
+            url = page.url(request, querystr={'highlight': query.highlight_re()},
+                           escape=0)
+            request.http_redirect(url)
+            raise MoinMoinNoFooter
 
     # send http headers
     request.http_headers()
@@ -108,3 +110,4 @@ def execute(pagename, request, fieldname='value', titlesearch=0):
     # End content and send footer
     request.write(request.formatter.endContent())
     wikiutil.send_footer(request, pagename)
+
