@@ -5,7 +5,7 @@
     @license: GNU GPL, see COPYING for details.
 """
 
-import re
+import re, os
 import xml.dom.minidom
 from xml.dom import Node
 
@@ -542,9 +542,9 @@ class convert_tree(visitor):
             #    self.text.append("\n")
 
     def process_br(self, node):
-            self.text.append('\n') # without this, std multi-line text below some heading misses a whitespace
-                                   # when it gets merged to float text, like word word wordword word word
-        
+            self.text.append(self.new_line) # without this, std multi-line text below some heading misses a whitespace
+                                            # when it gets merged to float text, like word word wordword word word
+
     def process_heading(self, node):
         text = self.node_list_text_only(node.childNodes).strip()
         if text:
@@ -609,7 +609,7 @@ class convert_tree(visitor):
                     raise ConvertError("Illegal list element %s" % i.localName)
         self.depth -= 1
         if self.depth == 0:
-            self.text.append("\n")
+            self.text.append(self.new_line)
 
     def process_list(self, node):
         self.depth += 1
@@ -628,24 +628,41 @@ class convert_tree(visitor):
                     raise ConvertError("Illegal list element %s" % i.localName)
         self.depth -= 1
         if self.depth == 0:
-            self.text.append("\n")
+            self.text.append(self.new_line)
 
     process_ul = process_list
     process_ol = process_list
 
+    def empty_paragraph_queue(self, nodelist, indent, need_indent):
+        if need_indent:
+            self.text.append(indent)
+        for i in nodelist:
+            if i.nodeType == Node.ELEMENT_NODE:
+                self.process_inline(i)
+            elif i.nodeType == Node.TEXT_NODE:
+                self.text.append(i.data.strip('\n').replace('\n', ' '))
+        self.text.append(self.new_line)
+        del nodelist[:]
+
     def process_list_item(self, node, indent):
         found = False
-        first_child = True
+        need_indent = False
+        pending = []
         for i in node.childNodes:
             name = i.localName
+
+            if name in ('p', 'pre', 'ol', 'ul', 'dl', 'table',) and pending:
+                self.empty_paragraph_queue(pending, indent, need_indent)
+                need_indent = True
+                
             if name == 'p':
-                if not first_child:
+                if need_indent:
                     self.text.append(indent)
                 self.process_paragraph_item(i)
-                self.text.append("\n")
+                self.text.append(self.new_line)
                 found = True
             elif name == 'pre':
-                if not first_child:
+                if need_indent:
                     self.text.append(indent)
                 self.process_preformatted_item(i)
                 found = True
@@ -656,17 +673,18 @@ class convert_tree(visitor):
                 self.process_dl(i)
                 found = True
             elif name == 'table':
-                if not first_child:
+                if need_indent:
                     self.text.append(indent)
                 self.process_table(i)
                 found = True
-            #else:
-            #    self.process_inline(i)
-            first_child = False
-                
-        if not found:
-            self.process_paragraph_item(node)
-            self.text.append("\n")
+            else:
+                pending.append(i)
+
+            if found:
+                need_indent = True
+
+        if pending:
+            self.empty_paragraph_queue(pending, indent, need_indent)
 
     def process_blockquote(self, node):
         # XXX this does not really work. e.g.:
@@ -707,7 +725,7 @@ class convert_tree(visitor):
 
     def process_inline(self, node):
         if node.nodeType == Node.TEXT_NODE:
-            self.text.append(node.data.strip('\n'))
+            self.text.append(node.data.strip('\n').replace('\n',' '))
             return
 
         name = node.localName # can be None for DOM Comment nodes
@@ -754,7 +772,7 @@ class convert_tree(visitor):
             if i.nodeType == Node.ELEMENT_NODE:
                 self.process_inline(i)
             elif i.nodeType == Node.TEXT_NODE:
-                self.text.append(i.data.strip('\n'))
+                self.text.append(i.data.strip('\n').replace('\n',' '))
         if command_close:
             command = command_close
         self.text.append(command)
@@ -789,18 +807,18 @@ class convert_tree(visitor):
 
     def process_p(self, node):
         self.process_paragraph_item(node)
-        self.text.append("\n\n")
+        self.text.append("\n\n") # do not use self.new_line here!
 
     def process_paragraph_item(self, node):
         for i in node.childNodes:
             if i.nodeType == Node.ELEMENT_NODE:
                 self.process_inline(i)
             elif i.nodeType == Node.TEXT_NODE:
-                self.text.append(i.data.strip('\n'))
+                self.text.append(i.data.strip('\n').replace('\n', ' '))
 
     def process_pre(self, node):
         self.process_preformatted_item(node)
-        self.text.append("\n")
+        self.text.append(self.new_line)
 
     def process_preformatted_item(self, node):
         if node.hasAttribute("class"):
@@ -813,7 +831,7 @@ class convert_tree(visitor):
                     self.text.append(i.data)
                     #print "'%s'" % i.data
                 elif i.localName == 'br':
-                    self.text.append("\n")
+                    self.text.append(self.new_line)
                 else:
                     pass
                     #print i.localName
@@ -824,11 +842,11 @@ class convert_tree(visitor):
                     self.text.append(i.data)
                     #print "'%s'" % i.data
                 elif i.localName == 'br':
-                    self.text.append("\n")
+                    self.text.append(self.new_line)
                 else:
                     pass
                     #print i.localName
-            self.text.append("}}}\n")
+            self.text.extend(["}}}", self.new_line])
 
     _alignment = {"left" : "(",
                   "center" : ":",
@@ -866,7 +884,9 @@ class convert_tree(visitor):
             result.append('tablealign="%s"' % value)
         if node.hasAttribute("style"):
             result.append('tablestyle="%s"' % node.getAttribute("style"))
-        return " ".join(result)
+        if node.hasAttribute("class"):
+            result.append('tableclass="%s"' % node.getAttribute("class"))
+        return " ".join(result).strip()
 
     def _row_style(self, node):
         result = []
@@ -882,11 +902,9 @@ class convert_tree(visitor):
                 result.append('rowbgcolor="%s"' % value)
         if node.hasAttribute("style"):
             result.append('rowstyle="%s"' % node.getAttribute("style"))
-        return "".join(result)
+        return " ".join(result).strip()
 
     def _cell_style(self, node):
-        align = ""
-
         if node.hasAttribute("rowspan"):
             rowspan = ("|%s" % node.getAttribute("rowspan"))
         else:
@@ -899,8 +917,8 @@ class convert_tree(visitor):
 
         spanning = rowspan or colspan > 1
         
+        align = ""
         result = []
-
         if  node.hasAttribute("bgcolor"):
             value = node.getAttribute("bgcolor")
             match = re.match(r"rgb\((\d+),\s*(\d+),\s*(\d+)\)", value)
@@ -939,11 +957,10 @@ class convert_tree(visitor):
         if align:
             result[0:0] = "%s" % align
         result.append(rowspan)
-        result = "".join(result)
-        #print result
-        return result
+        return " ".join(result).strip()
 
     def process_table(self, node, style=""):
+        self.text.append(self.new_line)
         self.new_table = True
         style += self._table_style(node)
         for i in node.childNodes:
@@ -961,7 +978,7 @@ class convert_tree(visitor):
                     raise ConvertError("process_table: Don't support %s element" % name)
             #else:
             #    raise ConvertError("Unexpected node: %r" % i)
-        self.text.append("\n")
+        self.text.append(self.new_line)
 
     def process_caption(self, table, node, style=""):
         # get first row
@@ -989,7 +1006,7 @@ class convert_tree(visitor):
             colspan = 1
         text = self.node_list_text_only(node.childNodes).replace('\n', ' ').strip()
         if text:
-            self.text.append("%s'''%s%s'''||\n" % ('||' * colspan, style, text))
+            self.text.extend(["%s'''%s%s'''||" % ('||' * colspan, style, text), self.new_line])
 
     def process_table_data(self, node, style=""):
         if node.hasAttribute("colspan"):
@@ -1015,7 +1032,7 @@ class convert_tree(visitor):
                     self.process_inline(i)
                     found = True
                 elif i.nodeType == Node.TEXT_NODE:
-                    data = i.data.strip('\n')
+                    data = i.data.strip('\n').replace('\n',' ')
                     if data:
                         found = True
                         self.text.append(data)
@@ -1036,7 +1053,7 @@ class convert_tree(visitor):
                     style = ""
                 else:
                     raise ConvertError("process_table_record: Don't support %s element" % name)
-        self.text.append("||\n")
+        self.text.extend(["||", self.new_line])
 
     def process_a(self, node):
         scriptname = self.request.getScriptname()
@@ -1166,23 +1183,31 @@ class convert_tree(visitor):
                     pass #print name, data, filename, alt
             raise ConvertError("Unknown smiley icon '%s'" % filename)
         # Image URL
-        elif src and src.startswith("http:") and wikiutil.isPicture(src):
+        elif src and src.startswith("http") and wikiutil.isPicture(src): # matches http: and https: !
             self.text.extend([self.white_space, src, self.white_space])
         else:
             raise ConvertError("Strange image src: '%s'" % src)
 
 
-def parse(text):
+def parse(request, text):
     text = u'<?xml version="1.0"?>%s%s' % (dtd, text)
     text = text.encode(config.charset)
     try:
         return xml.dom.minidom.parseString(text)
     except xml.parsers.expat.ExpatError, msg:
-        raise ConvertError('ExpatError: %s' % msg)
+        # this sometimes crashes when it should not, so save the stuff to analyze it:
+        logname = os.path.join(request.cfg.data_dir, "expaterror.log")
+        f = file(logname, "w")
+        f.write(text)
+        f.write("\n" + "-"*80 + "\n" + msg)
+        f.close()
+        raise ConvertError('ExpatError: %s (see dump in %s)' % (msg, logname))
 
 def convert(request, pagename, text):
     text = u"<page>%s</page>" % text
-    tree = parse(text)
+    tree = parse(request, text)
     strip_whitespace().do(tree)
-    return convert_tree(request, pagename).do(tree)
+    text = convert_tree(request, pagename).do(tree)
+    text = '\n'.join([s.rstrip() for s in text.splitlines()] + ['']) # remove trailing blanks
+    return text
 
