@@ -29,6 +29,9 @@ _blocks = Set(['dd', 'div', 'dl', 'dt', 'form', 'h1', 'h2', 'h3', 'h4', 'h5', 'h
 _self_closing_tags = Set(['area', 'base', 'br', 'col', 'frame', 'hr', 'img', 'input',
                           'isindex', 'link', 'meta', 'param'])
 
+# We only open those tags and let the browser auto-close them:
+_auto_closing_tags = Set(['p'])
+
 # These are the elements which generally should cause an increase in the
 # indention level in the html souce code.
 _indenting_tags = Set(['ol', 'ul', 'dl', 'li', 'dt', 'dd', 'tr', 'td'])
@@ -51,7 +54,8 @@ _common_attributes = Set(['accesskey', 'class', 'dir', 'disabled', 'id', 'lang',
 
 
 def rewrite_attribute_name(name, default_namespace='html'):
-    """Takes an attribute name and tries to make it HTML correct.
+    """
+    Takes an attribute name and tries to make it HTML correct.
 
     This function takes an attribute name as a string, as it may be
     passed in to a formatting method using a keyword-argument syntax,
@@ -211,7 +215,7 @@ class Formatter(FormatterBase):
             defined, will return attributes only if the current lang is
             different from the content lang.
         @rtype: dict
-        @retrun: language attributes
+        @return: language attributes
         """
         if not lang:
             lang = self.request.current_lang
@@ -230,7 +234,7 @@ class Formatter(FormatterBase):
 
         @param attr: dict containing keys and values
         @param allowed_attrs: A list of allowable attribute names
-        @param **kw: other arbitrary attributes expressed as keyword arguments.
+        @param kw: other arbitrary attributes expressed as keyword arguments.
         @rtype: string
         @return: formated attributes or empty string
 
@@ -325,10 +329,8 @@ class Formatter(FormatterBase):
         @rtype: string ?
         @return: open tag with attributes as a string
         """
-        is_self_closing = ''
-        if tag in _self_closing_tags:
-            # Don't expect a closing tag later on.
-            is_self_closing = ' /'
+        # If it is self-closing, then don't expect a closing tag later on.
+        is_self_closing = (tag in _self_closing_tags) and ' /' or ''
 
         if tag in _blocks:
             # Block elements
@@ -370,7 +372,7 @@ class Formatter(FormatterBase):
         @rtype: string
         @return: closing tag as a string
         """
-        if tag in _self_closing_tags:
+        if tag in _self_closing_tags or tag in _auto_closing_tags:
             # This tag was already closed
             tagstr = ''
         elif tag in _blocks:
@@ -483,7 +485,6 @@ class Formatter(FormatterBase):
             See wikiutil.link_tag() for possible keyword parameters.
         """
         apply(FormatterBase.pagelink, (self, on, pagename, page), kw)
-        self.pagelink_preclosed = False
         if page is None:
             page = Page(self.request, pagename, formatter=self);
         if self.request.user.show_nonexist_qm and on and not page.exists():
@@ -492,6 +493,7 @@ class Formatter(FormatterBase):
                     self.text("?") +
                     page.link_to(self.request, on=0, **kw))
         elif not on and self.pagelink_preclosed:
+            self.pagelink_preclosed = False
             return ""
         else:
             return page.link_to(self.request, on=on, **kw)
@@ -500,85 +502,74 @@ class Formatter(FormatterBase):
         """
         @keyword title: override using the interwiki wikiname as title
         """
-        if not on:
-            return self.url(0)
         wikitag, wikiurl, wikitail, wikitag_bad = wikiutil.resolve_wiki(self.request, '%s:%s' % (interwiki, pagename))
         wikiurl = wikiutil.mapURL(self.request, wikiurl)
         if wikitag == 'Self': # for own wiki, do simple links
-            if wikitail.find('#') > -1:
-                wikitail, kw['anchor'] = wikitail.split('#', 1)
-            wikitail = wikiutil.url_unquote(wikitail)
-            try: # XXX this is the only place where we access self.page - do we need it? Crashes silently on actions!
-                return apply(self.pagelink, (on, wikiutil.AbsPageName(self.request, self.page.page_name, wikitail)), kw)
-            except:
-                return apply(self.pagelink, (on, wikitail), kw)
-        else: # return InterWiki hyperlink
-            href = wikiutil.join_wiki(wikiurl, wikitail)
-            if wikitag_bad:
-                html_class = 'badinterwiki'
+            if on:
+                if wikitail.find('#') > -1:
+                    wikitail, kw['anchor'] = wikitail.split('#', 1)
+                wikitail = wikiutil.url_unquote(wikitail)
+                try: # XXX this is the only place where we access self.page - do we need it? Crashes silently on actions!
+                    return apply(self.pagelink, (on, wikiutil.AbsPageName(self.request, self.page.page_name, wikitail)), kw)
+                except:
+                    return apply(self.pagelink, (on, wikitail), kw)
             else:
-                html_class = 'interwiki'
-            title = kw.get('title', wikitag)
-            return self.url(1, href, title=title, do_escape=1, css=html_class) # interwiki links with umlauts
+                return self.pagelink(0)
+        else: # return InterWiki hyperlink
+            if on:
+                href = wikiutil.join_wiki(wikiurl, wikitail)
+                if wikitag_bad:
+                    html_class = 'badinterwiki'
+                else:
+                    html_class = 'interwiki'
+                title = kw.get('title', wikitag)
+                return self.url(1, href, title=title, do_escape=1, css=html_class) # interwiki links with umlauts
+            else:
+                return self.url(0)
 
     def url(self, on, url=None, css=None, do_escape=0, **kw):
-        """ Inserts an <a> element.
-
-            Call once with on=1 to start the link, and again with on=0
-            to end it (no other arguments are needed when on==0).
-
-            do_escape: XXX doesn't work yet
-
-            Keyword params:
-                url - the URL to link to; will go through Wiki URL mapping.
-                css - a space-separated list of CSS classes
-                attrs -  just include this string verbatim inside
-                         the <a> element; can be used for arbitrary attrs;
-                         all escaping and quoting is the caller's responsibility.
-
-            Note that the 'attrs' keyword argument is for backwards compatibility
-            only.  It should not be used for new code--instead just pass
-            any attributes in as separate keyword arguments.
         """
-        if not on:
-            return self._close('a')
-        attrs = self._langAttr()
+        Inserts an <a> element.
 
-        # Handle the URL mapping
-        if url is None and kw.has_key('href'):
-            url = kw['href']
-            del kw['href']
-        if url is not None:
-            url = wikiutil.mapURL(self.request, url)
-            
-            # TODO just calling url_quote does not work, as it will also quote "http:" to "http%xx" X)
-            if 0: # do_escape: # protocol and server part must not get quoted, path should get quoted
-                url = wikiutil.url_quote(url)
-            
-            attrs['href'] = url
+        Call once with on=1 to start the link, and again with on=0
+        to end it (no other arguments are needed when on==0).
 
-        if css:
-            attrs['class'] = css
-        
-        if kw.has_key('attrs'):
-            # for backwards compatibility, raw pre-formated attribute string
-            extra_attrs = kw['attrs']
-            del kw['attrs']
-        else:
-            extra_attrs = None
+        do_escape: XXX doesn't work yet
 
-        # create link
+        Keyword params:
+            url - the URL to link to; will go through Wiki URL mapping.
+            css - a space-separated list of CSS classes
+            attrs -  just include this string verbatim inside
+                     the <a> element; can be used for arbitrary attrs;
+                     all escaping and quoting is the caller's responsibility.
+
+        Note that the 'attrs' keyword argument is for backwards compatibility
+        only.  It should not be used for new code -- instead just pass
+        any attributes in as separate keyword arguments.
+
+        1.5.3: removed ugly "attrs" keyword argument handling code
+        """
         if on:
-            str = self._open('a', attr=attrs, **kw)
-            if extra_attrs:
-                # insert this into the tag verbatim (messy)
-                if str[-2:] == '/>':
-                    str = '%s %s />' % (str[:-2], extra_attrs)
-                else:
-                    str = '%s %s>' % (str[:-1], extra_attrs)
+            attrs = self._langAttr()
+
+            # Handle the URL mapping
+            if url is None and kw.has_key('href'):
+                url = kw['href']
+                del kw['href']
+            if url is not None:
+                url = wikiutil.mapURL(self.request, url)
+                # TODO just calling url_quote does not work, as it will also quote "http:" to "http%xx" X)
+                if 0: # do_escape: # protocol and server part must not get quoted, path should get quoted
+                    url = wikiutil.url_quote(url)
+                attrs['href'] = url
+
+            if css:
+                attrs['class'] = css
+            
+            markup = self._open('a', attr=attrs, **kw)
         else:
-            str = self._close('a')
-        return str
+            markup = self._close('a')
+        return markup
 
     def anchordef(self, id):
         """Inserts an invisible element used as a link target.
@@ -731,7 +722,7 @@ class Formatter(FormatterBase):
                                      edit_link,
                                      self.image(alt=url,
                                                 src=AttachFile.getAttachUrl(pagename, filename, self.request, addts=1), css="drawing"),
-                                     attrs='title="%s"' % (_('Edit drawing %(filename)s') % {'filename': self.text(fname)}))
+                                     title="%s" % (_('Edit drawing %(filename)s') % {'filename': self.text(fname)}))
         
     
     # def attachment_inlined(self, url, text, **kw):
@@ -947,6 +938,7 @@ function togglenumber(did, nstart, nstep) {
         Call once with on=1 to start the region, and a second time
         with on=0 to end it.
         """
+        _ = self.request.getText
         res = []
         ci = self.request.makeUniqueID('CA-%s_%03d' % (code_id, self._code_area_num))
         if on:
@@ -969,9 +961,10 @@ function togglenumber(did, nstart, nstep) {
                 toggleLineNumbersLink = r'''
 <script type="text/javascript">
 document.write('<a href="#" onclick="return togglenumber(\'%s\', %d, %d);" \
-                class="codenumbers">Toggle line numbers<\/a>');
+                class="codenumbers">%s<\/a>');
 </script>
-''' % (self._code_area_state[0], self._code_area_state[2], self._code_area_state[3])
+''' % (self._code_area_state[0], self._code_area_state[2], self._code_area_state[3],
+       _("Toggle line numbers"))
                 res.append(toggleLineNumbersLink)
 
             # Open pre - using left to right always!
@@ -1245,14 +1238,14 @@ document.write('<a href="#" onclick="return togglenumber(\'%s\', %d, %d);" \
 
         @param attrs: attribute dict
         @param prefix: used in wiki table attributes
-        @rtyp: dict
+        @rtype: dict
         @return: valid table attributes
         """
         if not attrs:
             return {}
 
         result = {}
-        s = "" # we collect synthesized style in s
+        s = [] # we collect synthesized style in s
         for key, val in attrs.items():
             # Ignore keys that don't start with prefix
             if prefix and key[:len(prefix)] != prefix:
@@ -1261,24 +1254,31 @@ document.write('<a href="#" onclick="return togglenumber(\'%s\', %d, %d);" \
             val = val.strip('"')
             # remove invalid attrs from dict and synthesize style
             if key == 'width':
-                s += "width: %s;" % val
+                s.append("width: %s" % val)
             elif key == 'height':
-                s += "height: %s;" % val
+                s.append("height: %s" % val)
             elif key == 'bgcolor':
-                s += "background-color: %s;" % val
+                s.append("background-color: %s" % val)
             elif key == 'align':
-                s += "text-align: %s;" % val
+                s.append("text-align: %s" % val)
             elif key == 'valign':
-                s += "vertical-align: %s;" % val
+                s.append("vertical-align: %s" % val)
             # Ignore unknown keys
             if key not in self._allowed_table_attrs[prefix]:
                 continue
             result[key] = val
-        if s:
-            if result.has_key('style'):
-                result['style'] += s
-            else:
-                result['style'] = s
+        st = result.get('style', '').split(';')
+        st = '; '.join(st + s)
+        st = st.strip(';')
+        st = st.strip()
+        if not st:
+            try:
+                del result['style'] # avoid empty style attr
+            except:
+                pass
+        else:
+            result['style'] = st
+        #self.request.log("_checkTableAttr returns %r" % result)
         return result
 
 
