@@ -99,7 +99,7 @@ class EditLogLine:
             title = wikiutil.escape(aliasname + title)
             text = (request.formatter.interwikilink(1, title=title, *info) +
                     request.formatter.text(name) +
-                    request.formatter.interwikilink(0))
+                    request.formatter.interwikilink(0, title=title, *info))
         elif kind == 'email':
             name = self._usercache[self.userid].name
             aliasname = self._usercache[self.userid].aliasname
@@ -150,7 +150,8 @@ class EditLog(LogFile):
                 
             try:
                 hostname = socket.gethostbyaddr(host)[0]
-            except socket.error:
+                hostname = unicode(hostname, config.charset)
+            except (socket.error, UnicodeError), err:
                 hostname = host
 
             remap_chars = {u'\t': u' ', u'\r': u' ', u'\n': u' ',}
