@@ -44,14 +44,14 @@ You have to use four spaces at the beginning of the line mostly.
     return module, mtime
 
 
-def _url_re():
+def _url_re_list():
     """ Return url matching regular expression
 
-    Import wikis list from farmconfig on the first call and compile a
-    regex. Later then return the cached regex.
+    Import wikis list from farmconfig on the first call and compile the
+    regexes. Later then return the cached regex list.
 
-    @rtype: compiled re object
-    @return: url to wiki config  matching re
+    @rtype: list of tuples of (name, compiled re object)
+    @return: url to wiki config name matching list
     """
     global _url_re_cache, _farmconfig_mtime
     if _url_re_cache is None:
@@ -60,12 +60,13 @@ def _url_re():
         except ImportError:
             # Default to wikiconfig for all urls.
             _farmconfig_mtime = 0
-            _url_re_cache = re.compile(r'(?P<wikiconfig>.)')
+            _url_re_cache = [('wikiconfig', re.compile(r'.')), ] # matches everything
         else:
             try:
-                pattern = '|'.join([r'(?P<%s>%s)' % (name, regex)
-                                    for name, regex in farmconfig.wikis])
-                _url_re_cache = re.compile(pattern)
+                cache = []
+                for name, regex in farmconfig.wikis:
+                    cache.append((name, re.compile(regex)))
+                _url_re_cache = cache
             except AttributeError:
                 msg = """
 Missing required 'wikis' list in 'farmconfig.py'.
@@ -121,18 +122,18 @@ use the wikiconfig.py file from the distribution.
 
 def _getConfigName(url):
     """ Return config name for url or raise """
-    match = _url_re().match(url)
-    if not (match and match.groups()):
-        msg = '''
+    for name, regex in _url_re_list():
+        match = regex.match(url)
+        if match:
+            return name
+    # nothing matched
+    msg = '''
 Could not find a match for url: "%(url)s".
 
 Check your URL regular expressions in the "wikis" list in
 "farmconfig.py". 
 ''' % {'url': url}
-        raise error.ConfigurationError(msg)    
-    for name, value in match.groupdict().items():
-        if value: break
-    return name
+    raise error.ConfigurationError(msg)    
 
 
 def getConfig(url):
@@ -206,6 +207,36 @@ class DefaultConfig:
     editor_default = 'text' # which editor is called when nothing is specified
     editor_ui = 'freechoice' # which editor links are shown on user interface
     editor_force = False
+    editor_quickhelp = { # editor markup hints quickhelp 
+        'wiki': _("""\
+ Emphasis:: [[Verbatim('')]]''italics''[[Verbatim('')]]; [[Verbatim(''')]]'''bold'''[[Verbatim(''')]]; [[Verbatim(''''')]]'''''bold italics'''''[[Verbatim(''''')]]; [[Verbatim('')]]''mixed ''[[Verbatim(''')]]'''''bold'''[[Verbatim(''')]] and italics''[[Verbatim('')]]; [[Verbatim(----)]] horizontal rule.
+ Headings:: [[Verbatim(=)]] Title 1 [[Verbatim(=)]]; [[Verbatim(==)]] Title 2 [[Verbatim(==)]]; [[Verbatim(===)]] Title 3 [[Verbatim(===)]];   [[Verbatim(====)]] Title 4 [[Verbatim(====)]]; [[Verbatim(=====)]] Title 5 [[Verbatim(=====)]].
+ Lists:: space and one of: * bullets; 1., a., A., i., I. numbered items; 1.#n start numbering at n; space alone indents.
+ Links:: [[Verbatim(JoinCapitalizedWords)]]; [[Verbatim(["brackets and double quotes"])]]; url; [url]; [url label].
+ Tables:: || cell text |||| cell text spanning 2 columns ||;    no trailing white space allowed after tables or titles.
+
+(!) For more help, see HelpOnEditing or SyntaxReference.
+"""),
+        'rst': _("""\
+Emphasis: <i>*italic*</i> <b>**bold**</b> ``monospace``<br/>
+<br/><pre>
+Headings: Heading 1  Heading 2  Heading 3
+          =========  ---------  ~~~~~~~~~
+
+Horizontal rule: ---- 
+Links: TrailingUnderscore_ `multi word with backticks`_ external_ 
+
+.. _external: http://external-site.net/foo/
+
+Lists: * bullets; 1., a. numbered items.
+</pre>
+<br/>
+(!) For more help, see the 
+<a href="http://docutils.sourceforge.net/docs/user/rst/quickref.html">
+reStructuredText Quick Reference
+</a>.
+"""),
+    }
     edit_locking = 'warn 10' # None, 'warn <timeout mins>', 'lock <timeout mins>'
     edit_rows = 20
                 
@@ -299,9 +330,11 @@ class DefaultConfig:
     
     surge_action_limits = { # allow max. <count> <action> requests per <dt> secs
         # action: (count, dt)
-        'show': (20, 60),
+        'all': (30, 30),
+        'show': (30, 60),
+        'recall': (5, 60),
         'raw': (20, 40),  # some people use this for css
-        'AttachFile': (60, 60),
+        'AttachFile': (90, 60),
         'diff': (30, 60),
         'fullsearch': (5, 60),
         'edit': (10, 120),
@@ -321,7 +354,7 @@ class DefaultConfig:
     # and receive a FORBIDDEN for anything except viewing a page
     ua_spiders = ('archiver|cfetch|crawler|curl|gigabot|google|holmes|htdig|httrack|httpunit|jeeves|larbin|leech|'
                   'linkbot|linkmap|linkwalk|mercator|mirror|msnbot|nutbot|omniexplorer|puf|robot|scooter|'
-                  'search|sherlock|sitecheck|spider|teleport|voyager|wget')
+                  'search|sherlock|sitecheck|spider|teleport|voyager|webreaper|wget')
 
     # Wiki identity
     sitename = u'Untitled Wiki'
@@ -434,10 +467,14 @@ class DefaultConfig:
         # Try to decode certain names which allow unicode
         self._decode()
 
-        # Make sure directories are accessible
         self._check_directories()
 
-        # Load plugin module
+        if not isinstance(self.superuser, list):
+            msg = """The superuser</b> setting in your wiki configuration is not a list
+                     (e.g. ['Sample User', 'AnotherUser']).
+                     Please change it in your wiki configuration and try again."""
+            raise error.ConfigurationError(msg)
+        
         self._loadPluginModule()
 
         # Preparse user dicts
