@@ -120,8 +120,9 @@ class Parser:
         self.is_u = 0
         self.is_strike = 0
         self.lineno = 0
-        self.in_li = 0
-        self.in_dd = 0
+        self.in_list = 0 # between <ul/ol/dl> and </ul/ol/dl>
+        self.in_li = 0 # between <li> and </li>
+        self.in_dd = 0 # between <dd> and </dd>
         self.in_pre = 0
         self.in_table = 0
         self.is_big = False
@@ -196,7 +197,7 @@ class Parser:
               
         return (self.formatter.interwikilink(1, tag, tail) + 
                 self.formatter.text(text) +
-                self.formatter.interwikilink(0))
+                self.formatter.interwikilink(0, tag, tail))
 
     def attachment(self, url_and_text, **kw):
         """ This gets called on attachment URLs.
@@ -315,7 +316,6 @@ class Parser:
 
     def _rule_repl(self, word):
         """Handle sequences of dashes."""
-        ##self.inhibit_p = 1
         result = self._undent() + self._closeP()
         if len(word) <= 4:
             result = result + self.formatter.rule()
@@ -350,7 +350,7 @@ class Parser:
         anchor = ""
         if len(parts)==2:
             word, anchor = parts
-            
+
         return (self.formatter.pagelink(1, word, anchor=anchor) +
                 self.formatter.text(text) +
                 self.formatter.pagelink(0, word))
@@ -462,7 +462,7 @@ class Parser:
     def _indent_repl(self, match):
         """Handle pure indentation (no - * 1. markup)."""
         result = []
-        if not self.in_li:
+        if not (self.in_li or self.in_dd):
             self._close_item(result)
             self.in_li = 1
             css_class = None
@@ -497,20 +497,16 @@ class Parser:
         """Handle numbered lists."""
         return self._li_repl(match)
 
-
     def _dl_repl(self, match):
         """Handle definition lists."""
         result = []
         self._close_item(result)
-        #self.inhibit_p = 1
         self.in_dd = 1
         result.extend([
             self.formatter.definition_term(1),
             self.formatter.text(match[1:-3].lstrip(' ')),
             self.formatter.definition_term(0),
             self.formatter.definition_desc(1),
-            ## CHANGE: no automatic paragraph
-            ##self.formatter.paragraph(1)
         ])
         return ''.join(result)
 
@@ -528,16 +524,7 @@ class Parser:
         if self._indent_level() != new_level and self.in_table:
             close.append(self.formatter.table(0))
             self.in_table = 0
-        #    #self._close_item(close)
-        #else:
-        #    if not self.line_was_empty:
-        #        self.inhibit_p = 1
-    
-        # Close lists while char-wise indent is greater than the current one
-        #while ((self._indent_level() > new_level) or
-        #       ( new_level and
-        #        (self._indent_level() == new_level) and
-        #        (self.list_types[-1]) != list_type)):
+        
         while self._indent_level() > new_level:
             self._close_item(close)
             if self.list_types[-1] == 'ol':
@@ -548,14 +535,9 @@ class Parser:
                 tag = self.formatter.bullet_list(0)
             close.append(tag)
 
-            del(self.list_indents[-1])
-            del(self.list_types[-1])
+            del self.list_indents[-1]
+            del self.list_types[-1]
             
-            #if new_level:
-            #    self.inhibit_p = 1
-            #else:
-            #    self.inhibit_p = 0
-
             if self.list_types: # we are still in a list
                 if self.list_types[-1] == 'dl':
                     self.in_dd = 1
@@ -579,8 +561,6 @@ class Parser:
             open.append(tag)
             
             self.first_list_item = 1
-            ## Maybe this prevent p creation in lists?
-            ##self.inhibit_p = 1
             self.in_li = 0
             self.in_dd = 0
             
@@ -589,9 +569,7 @@ class Parser:
             close[0:0] = [self.formatter.table(0)]
             self.in_table = 0
         
-        ## Maybe this prevent p creation in lists?
-        ##self.inhibit_p = bool(self.list_types)
-
+        self.in_list = self.list_types != []
         return ''.join(close) + ''.join(open)
 
 
@@ -638,7 +616,14 @@ class Parser:
 
         # extension for special table markup
         def table_extension(key, parser, attrs, wiki_parser=self):
+            """ returns: tuple (found_flag, msg)
+                found_flag: whether we found something and were able to process it here
+                  true for special stuff like 100% or - or #AABBCC
+                  false for style xxx="yyy" attributes
+                msg: "" or an error msg
+            """
             _ = wiki_parser._
+            found = False
             msg = ''
             if key[0] in "0123456789":
                 token = parser.get_token()
@@ -653,6 +638,7 @@ class Parser:
                         msg = _('Expected an integer "%(key)s" before "%(token)s"') % {
                             'key': key, 'token': token}
                     else:
+                        found = True
                         attrs['width'] = '"%s%%"' % key
             elif key == '-':
                 arg = parser.get_token()
@@ -662,6 +648,7 @@ class Parser:
                     msg = _('Expected an integer "%(arg)s" after "%(key)s"') % {
                         'arg': arg, 'key': key}
                 else:
+                    found = True
                     attrs['colspan'] = '"%s"' % arg
             elif key == '|':
                 arg = parser.get_token()
@@ -671,16 +658,22 @@ class Parser:
                     msg = _('Expected an integer "%(arg)s" after "%(key)s"') % {
                         'arg': arg, 'key': key}
                 else:
+                    found = True
                     attrs['rowspan'] = '"%s"' % arg
             elif key == '(':
+                found = True
                 attrs['align'] = '"left"'
             elif key == ':':
+                found = True
                 attrs['align'] = '"center"'
             elif key == ')':
+                found = True
                 attrs['align'] = '"right"'
             elif key == '^':
+                found = True
                 attrs['valign'] = '"top"'
             elif key == 'v':
+                found = True
                 attrs['valign'] = '"bottom"'
             elif key == '#':
                 arg = parser.get_token()
@@ -691,16 +684,15 @@ class Parser:
                     msg = _('Expected a color value "%(arg)s" after "%(key)s"') % {
                         'arg': arg, 'key': key}
                 else:
+                    found = True
                     attrs['bgcolor'] = '"#%s"' % arg
-            else:
-                msg = None
-            #print "key: %s\nattrs: %s" % (key, str(attrs))
-            return self.formatter.rawHTML(msg)
+            return found, self.formatter.rawHTML(msg)
 
         # scan attributes
         attr, msg = wikiutil.parseAttributes(self.request, attrdef, '>', table_extension)
-        if msg: msg = '<strong class="highlight">%s</strong>' % msg
-        #print attr
+        if msg:
+            msg = '<strong class="highlight">%s</strong>' % msg
+        #self.request.log("parseAttributes returned %r" % attr)
         return attr, msg
 
     def _tableZ_repl(self, word):
@@ -742,6 +734,7 @@ class Parser:
 
             # return the complete cell markup
             result.append(self.formatter.table_cell(1, attrs) + attrerr)         
+            result.append(self._line_anchordef())
             return ''.join(result) 
         else:
             return self.formatter.text(word)
@@ -751,8 +744,6 @@ class Parser:
         """Handle section headings."""
         import sha
 
-        ##self.inhibit_p = 1
-
         h = word.strip()
         level = 1
         while h[level:level+1] == '=':
@@ -814,7 +805,6 @@ class Parser:
         word = word.strip()
         if word == '{{{' and not self.in_pre:
             self.in_pre = 3
-            ##self.inhibit_p = 1
             return self._closeP() + self.formatter.preformatted(self.in_pre)
         elif word == '}}}' and self.in_pre:
             self.in_pre = 0
@@ -831,6 +821,11 @@ class Parser:
 
 
     def _comment_repl(self, word):
+        # if we are in a paragraph, we must close it so that normal text following
+        # in the line below the comment will reopen a new paragraph.
+        if self.formatter.in_p:
+            self.formatter.paragraph(0)
+        self.line_is_empty = 1 # markup following comment lines treats them as if they were empty
         return self.formatter.comment(word)
 
     def _closeP(self):
@@ -841,7 +836,7 @@ class Parser:
     def _macro_repl(self, word):
         """Handle macros ([[macroname]])."""
         macro_name = word[2:-2]
-        self.inhibit_p = 1 # 0 fixes UserPreferences, but makes new trouble!
+        self.inhibit_p = 0 # 1 fixes UserPreferences, 0 fixes paragraph formatting for macros
 
         # check for arguments
         args = None
@@ -857,8 +852,7 @@ class Parser:
     def scan(self, scan_re, line):
         """ Scans one line
         
-        Append text before match, invoke replace() with match, and 
-        add text after match.
+        Append text before match, invoke replace() with match, and add text after match.
         """
         result = []
         lastpos = 0
@@ -872,19 +866,22 @@ class Parser:
                 ###result.append(u'<span class="info">[add text before match: <tt>"%s"</tt>]</span>' % line[lastpos:match.start()])
                 
                 if not (self.inhibit_p or self.in_pre or self.formatter.in_p):
-                    result.append(self.formatter.paragraph(1, css_class="line879"))
+                    result.append(self.formatter.paragraph(1, css_class="line862"))
                 result.append(self.formatter.text(line[lastpos:match.start()]))
             
             # Replace match with markup
+            if not (self.inhibit_p or self.in_pre or self.formatter.in_p or
+                    self.in_table or self.in_list):
+                result.append(self.formatter.paragraph(1, css_class="line867"))
             result.append(self.replace(match))
             lastpos = match.end()
         
         ###result.append('<span class="info">[no match, add rest: <tt>"%s"<tt>]</span>' % line[lastpos:])
         
-        # No match: Add paragraph with the text of the line
-        if not (self.in_pre or self.inhibit_p or
+        # Add paragraph with the remainder of the line
+        if not (self.in_pre or self.in_li or self.in_dd or self.inhibit_p or
                 self.formatter.in_p) and lastpos < len(line):
-            result.append(self.formatter.paragraph(1, css_class="line886"))
+            result.append(self.formatter.paragraph(1, css_class="line874"))
         result.append(self.formatter.text(line[lastpos:]))
         return u''.join(result)
 
@@ -901,7 +898,7 @@ class Parser:
                     # Open p for certain types
                     if not (self.inhibit_p or self.formatter.in_p
                             or self.in_pre or (type in self.no_new_p_before)):
-                        result.append(self.formatter.paragraph(1, css_class="line903"))
+                        result.append(self.formatter.paragraph(1, css_class="line891"))
                     
                     # Get replace method and replece hit
                     replace = getattr(self, '_' + type + '_repl')
@@ -916,6 +913,13 @@ class Parser:
 
         return ""
 
+    def _line_anchordef(self):
+        if self.line_anchors and not self.line_anchor_printed:
+            self.line_anchor_printed = 1
+            return self.formatter.line_anchordef(self.lineno)
+        else:
+            return ''
+
     def format(self, formatter):
         """ For each line, scan through looking for magic
             strings, outputting verbatim any intervening text.
@@ -951,8 +955,9 @@ class Parser:
         # Main loop
         for line in self.lines:
             self.lineno += 1
-            if self.line_anchors:
-                self.request.write(self.formatter.line_anchordef(self.lineno))
+            self.line_anchor_printed = 0
+            if not self.in_table:
+                self.request.write(self._line_anchordef())
             self.table_rowstart = 1
             self.line_was_empty = self.line_is_empty
             self.line_is_empty = 0
@@ -1026,6 +1031,7 @@ class Parser:
                 if not line.strip():
                     if self.in_table:
                         self.request.write(self.formatter.table(0))
+                        self.request.write(self._line_anchordef())
                         self.in_table = 0
                     # CHANGE: removed check for not self.list_types
                     # p should close on every empty line
@@ -1087,6 +1093,7 @@ class Parser:
                     
                     # Close table
                     self.request.write(self.formatter.table(0))
+                    self.request.write(self._line_anchordef())
                     self.in_table = 0
                                             
             # Scan line, format and write
