@@ -7,7 +7,7 @@
     @license: GNU GPL, see COPYING for details.
 """
 
-import os, time, sys, cgi, StringIO
+import os, re, time, sys, cgi, StringIO
 import copy
 from MoinMoin import config, wikiutil, user, caching
 from MoinMoin.util import MoinMoinNoFooter, IsWin9x
@@ -112,6 +112,8 @@ class RequestBase(object):
             self.__dict__.update(properties)
             self._load_multi_cfg()
             
+            self.isSpiderAgent = self.check_spider()
+        
             # Set decode charsets.  Input from the user is always in
             # config.charset, which is the page charsets. Except
             # path_info, which may use utf-8, and handled by decodePagename.
@@ -142,10 +144,12 @@ class RequestBase(object):
             rootname = u''
             self.rootpage = Page(self, rootname, is_rootpage=1)
 
-            self.user = self.get_user()
+            self.user = self.get_user_from_form()
             
             if not self.query_string.startswith('action=xmlrpc'):
-                if self.surge_protect():
+                if not self.forbidden and self.isForbidden():
+                    self.makeForbidden403()
+                if not self.forbidden and self.surge_protect():
                     self.makeUnavailable503()
 
             from MoinMoin import i18n
@@ -170,9 +174,9 @@ class RequestBase(object):
         """ check if someone requesting too much from us """
         validuser = self.user.valid
         current_id = validuser and self.user.name or self.remote_addr
-        current_action = self.form.get('action', ['show'])[0]
         if not validuser and current_id.startswith('127.'): # localnet
             return False
+        current_action = self.form.get('action', ['show'])[0]
         
         limits = self.cfg.surge_action_limits
         default_limit = self.cfg.surge_action_limits.get('default', (30, 60))
@@ -183,30 +187,45 @@ class RequestBase(object):
         
         try:
             cache = caching.CacheEntry(self, 'surgeprotect', 'surge-log')
-            data = cache.content()
-            data = data.split("\n")
-            for line in data:
-                try:
-                    id, t, action, surge_indicator = line.split("\t")
-                    t = int(t)
-                    maxnum, dt = limits.get(action, default_limit)
-                    if t >= now - dt:
-                        events = surgedict.setdefault(id, copy.copy({}))
-                        timestamps = events.setdefault(action, copy.copy([]))
-                        timestamps.append((t, surge_indicator))
-                except StandardError, err:
-                    pass
-        
+            if cache.exists():
+                data = cache.content()
+                data = data.split("\n")
+                for line in data:
+                    try:
+                        id, t, action, surge_indicator = line.split("\t")
+                        t = int(t)
+                        maxnum, dt = limits.get(action, default_limit)
+                        if t >= now - dt:
+                            events = surgedict.setdefault(id, copy.copy({}))
+                            timestamps = events.setdefault(action, copy.copy([]))
+                            timestamps.append((t, surge_indicator))
+                    except StandardError, err:
+                        pass
+                
             maxnum, dt = limits.get(current_action, default_limit)
             events = surgedict.setdefault(current_id, copy.copy({}))
             timestamps = events.setdefault(current_action, copy.copy([]))
             surge_detected = len(timestamps) > maxnum
+
             surge_indicator = surge_detected and "!" or ""
             timestamps.append((now, surge_indicator))
             if surge_detected:
                 if len(timestamps) < maxnum*2:
                     timestamps.append((now + self.cfg.surge_lockout_time, surge_indicator)) # continue like that and get locked out
         
+            if current_action != 'AttachFile': # don't add AttachFile accesses to all or picture galleries will trigger SP
+                current_action = 'all' # put a total limit on user's requests
+                maxnum, dt = limits.get(current_action, default_limit)
+                events = surgedict.setdefault(current_id, copy.copy({}))
+                timestamps = events.setdefault(current_action, copy.copy([]))
+                surge_detected = surge_detected or len(timestamps) > maxnum
+            
+                surge_indicator = surge_detected and "!" or ""
+                timestamps.append((now, surge_indicator))
+                if surge_detected:
+                    if len(timestamps) < maxnum*2:
+                        timestamps.append((now + self.cfg.surge_lockout_time, surge_indicator)) # continue like that and get locked out
+        
             data = []
             for id, events in surgedict.items():
                 for action, timestamps in events.items():
@@ -479,25 +498,41 @@ class RequestBase(object):
         else:
             path, query = uri, ''
         return wikiutil.url_unquote(path, want_unicode=False), query        
-                
-    def get_user(self):
-        # try to get some values from the maybe present UserPreferences form,
-        # so auth methods can use it as their user interface for login
+
+    def get_user_from_form(self):
+        """ read the maybe present UserPreferences form and call get_user with the values """
         name = self.form.get('name', [None])[0]
         password = self.form.get('password', [None])[0]
         login = self.form.has_key('login')
         logout = self.form.has_key('logout')
-
-        for auth in self.cfg.auth:
-            user_obj, continue_flag = auth(self,
-                                           name=name, password=password,
-                                           login=login, logout=logout)
-            if not continue_flag:
-                break
+        u = self.get_user_default_unknown(name=name, password=password,
+                                          login=login, logout=logout,
+                                          user_obj=None)
+        return u
+    
+    def get_user_default_unknown(self, **kw):
+        """ call do_auth and if it doesnt return a user object, make some "Unknown User" """
+        user_obj = self.get_user_default_None(**kw)
         if user_obj is None:
             user_obj = user.User(self, auth_method="request:427")
         return user_obj
 
+    def get_user_default_None(self, **kw):
+        """ loop over auth handlers, return a user obj or None """
+        name = kw.get('name')
+        password = kw.get('password')
+        login = kw.get('login')
+        logout = kw.get('logout')
+        user_obj = kw.get('user_obj')
+        for auth in self.cfg.auth:
+            user_obj, continue_flag = auth(self,
+                                           name=name, password=password,
+                                           login=login, logout=logout,
+                                           user_obj=user_obj)
+            if not continue_flag:
+                break
+        return user_obj
+        
     def reset(self):
         """ Reset request state.
 
@@ -528,10 +563,11 @@ class RequestBase(object):
 
         @param theme_name: the name of the theme
         @type theme_name: str
-        @returns: 0 on success, 1 if user theme could not be loaded,
-                  2 if a hard fallback to modern theme was required.
         @rtype: int
         @return: success code
+                 0 on success
+                 1 if user theme could not be loaded,
+                 2 if a hard fallback to modern theme was required.
         """
         fallback = 0
         if theme_name == "<default>":
@@ -590,7 +626,7 @@ class RequestBase(object):
         """ Try to get pagename from the query string
         
         Support urls like http://netloc/script/?page_name. Allow
-        solving path_info encodig problems by calling with the page
+        solving path_info encoding problems by calling with the page
         name as a query.
         """
         pagename = wikiutil.url_unquote(self.query_string, want_unicode=False)
@@ -829,26 +865,40 @@ class RequestBase(object):
         """ Flush output stream.
         """
         raise NotImplementedError
-        
+
+    def check_spider(self):
+        """ check if the user agent for current request is a spider/bot """
+        isSpider = False
+        spiders = self.cfg.ua_spiders
+        if spiders:
+            ua = self.getUserAgent()
+            if ua:
+                isSpider = re.search(spiders, ua, re.I) is not None
+        return isSpider
+
     def isForbidden(self):
         """ check for web spiders and refuse anything except viewing """
         forbidden = 0
-        # we do not have a parsed query string here
-        # so we can just do simple matching
-        if ((self.query_string != '' or self.request_method != 'GET') and
-            self.query_string != 'action=rss_rc' and not
+        # we do not have a parsed query string here, so we can just do simple matching
+        qs = self.query_string
+        if ((qs != '' or self.request_method != 'GET') and
+            not 'action=rss_rc' in qs and
             # allow spiders to get attachments and do 'show'
-            (self.query_string.find('action=AttachFile') >= 0 and self.query_string.find('do=get') >= 0) and not
-            (self.query_string.find('action=show') >= 0)
+            not ('action=AttachFile' in qs and 'do=get' in qs) and
+            not 'action=show' in qs
             ):
-            from MoinMoin.util import web
-            forbidden = web.isSpiderAgent(self)
+            forbidden = self.isSpiderAgent
 
         if not forbidden and self.cfg.hosts_deny:
             ip = self.remote_addr
             for host in self.cfg.hosts_deny:
-                if ip == host or host[-1] == '.' and ip.startswith(host):
+                if host[-1] == '.' and ip.startswith(host):
                     forbidden = 1
+                    #self.log("hosts_deny (net): %s" % str(forbidden))
+                    break
+                if ip == host:
+                    forbidden = 1
+                    #self.log("hosts_deny (ip): %s" % str(forbidden))
                     break
         return forbidden
 
@@ -876,9 +926,9 @@ class RequestBase(object):
         Create the args from a standard cgi.FieldStorage or from given
         form. Each key contain a list of values.
 
-        @keyword form: a cgi.FieldStorage
+        @param form: a cgi.FieldStorage
         @rtype: dict
-        @return dict with form keys, each contains a list of values
+        @return: dict with form keys, each contains a list of values
         """
         if form is None:
             form = cgi.FieldStorage()
@@ -926,7 +976,7 @@ class RequestBase(object):
 
         Already qualified urls are returned unchanged.
 
-        @param uri: server rootted uri e.g /scriptname/pagename. It
+        @param uri: server rooted uri e.g /scriptname/pagename. It
             must start with a slash. Must be ascii and url encoded.
         """
         import urlparse
@@ -934,8 +984,8 @@ class RequestBase(object):
         if scheme:
             return uri
 
-        schema = ('http', 'https')[self.is_ssl]
-        result = "%s://%s%s" % (schema, self.http_host, uri)
+        scheme = ('http', 'https')[self.is_ssl]
+        result = "%s://%s%s" % (scheme, self.http_host, uri)
 
         # This might break qualified urls in redirects!
         # e.g. mapping 'http://netloc' -> '/'
@@ -978,11 +1028,12 @@ class RequestBase(object):
     def run(self):
         # Exit now if __init__ failed or request is forbidden
         if self.failed or self.forbidden:
+            #Don't sleep()! Seems to bind too much resources, so twisted will
+            #run out of threads, files, whatever (with low CPU load) and stop
+            #serving requests.
+            #if self.forbidden:
+            #    time.sleep(10) # let the sucker wait!
             return self.finish()
-        if self.isForbidden():
-            self.makeForbidden403()
-            if self.forbidden:
-                return self.finish()
 
         self.open_logs()
         _ = self.getText
@@ -1027,7 +1078,7 @@ class RequestBase(object):
                 msg = _("""Invalid user name {{{'%s'}}}.
 Name may contain any Unicode alpha numeric character, with optional one
 space between words. Group page name is not allowed.""") % self.user.name
-                self.deleteCookie()
+                self.user = self.get_user_default_unknown(name=self.user.name, logout=True)
                 page = wikiutil.getSysPage(self, 'UserPreferences')
                 page.send_page(self, msg=msg)
 
@@ -1076,9 +1127,7 @@ space between words. Group page name is not allowed.""") % self.user.name
                     msg = _("You are not allowed to do %s on this page.") % wikiutil.escape(action)
                     if not self.user.valid:
                         # Suggest non valid user to login
-                        login = wikiutil.getSysPage(self, 'UserPreferences')
-                        login = login.link_to(self, _('Login'))
-                        msg += _(" %s and try again.", formatted=0) % login
+                        msg += _(" %s and try again.", formatted=0) % _('Login') # XXX merge into 1 string after 1.5.3 release
                     self.page.send_page(self, msg=msg)
 
                 # Try action
@@ -1249,84 +1298,6 @@ space between words. Group page name is not allowed.""") % self.user.name
         # Set Pragma for http 1.0 caches
         # See http://www.cse.ohio-state.edu/cgi-bin/rfc/rfc2068.html#sec-14.32
         self.setHttpHeader('Pragma: no-cache')
-       
-    def setCookie(self):
-        """ Set cookie for the current user
-        
-        cfg.cookie_lifetime and the user 'remember_me' setting set the
-        lifetime of the cookie. lifetime in int hours, see table:
-        
-        value   cookie lifetime
-        ----------------------------------------------------------------
-         = 0    forever, ignoring user 'remember_me' setting
-         > 0    n hours, or forever if user checked 'remember_me'
-         < 0    -n hours, ignoring user 'remember_me' setting
-        """
-        # Calculate cookie maxage and expires
-        lifetime = int(self.cfg.cookie_lifetime) * 3600 
-        forever = 10*365*24*3600 # 10 years
-        now = time.time()
-        if not lifetime:
-            maxage = forever
-        elif lifetime > 0:
-            if self.user.remember_me:
-                maxage = forever
-            else:
-                maxage = lifetime
-        elif lifetime < 0:
-            maxage = (-lifetime)
-        expires = now + maxage
-        
-        # Set the cookie
-        from Cookie import SimpleCookie
-        c = SimpleCookie()
-        c['MOIN_ID'] = self.user.id
-        c['MOIN_ID']['max-age'] = maxage
-        if self.cfg.cookie_domain:
-            c['MOIN_ID']['domain'] = self.cfg.cookie_domain
-        if self.cfg.cookie_path:
-            c['MOIN_ID']['path'] = self.cfg.cookie_path
-        else:
-            c['MOIN_ID']['path'] = self.getScriptname()
-        # Set expires for older clients
-        c['MOIN_ID']['expires'] = self.httpDate(when=expires, rfc='850')        
-        self.setHttpHeader(c.output())
-
-        # Update the saved cookie, so other code works with new setup
-        self.saved_cookie = c.output()
-
-        # IMPORTANT: Prevent caching of current page and cookie
-        self.disableHttpCaching()
-
-    def deleteCookie(self):
-        """ Delete the user cookie by sending expired cookie with null value
-
-        According to http://www.cse.ohio-state.edu/cgi-bin/rfc/rfc2109.html#sec-4.2.2
-        Deleted cookie should have Max-Age=0. We also have expires
-        attribute, which is probably needed for older browsers.
-
-        Finally, delete the saved cookie and create a new user based on
-        the new settings.
-        """
-        # Set cookie
-        from Cookie import SimpleCookie
-        c = SimpleCookie()
-        c['MOIN_ID'] = ''
-        if self.cfg.cookie_domain:
-            c['MOIN_ID']['domain'] = self.cfg.cookie_domain
-        c['MOIN_ID']['path'] = self.getScriptname()
-        c['MOIN_ID']['max-age'] = 0
-        # Set expires to one year ago for older clients
-        yearago = time.time() - (3600 * 24 * 365)
-        c['MOIN_ID']['expires'] = self.httpDate(when=yearago, rfc='850')
-        self.setHttpHeader(c.output())
-
-        # Update saved cookie and set new unregistered user
-        self.saved_cookie = ''
-        self.user = user.User(self, auth_method="request:1245")
-
-        # IMPORTANT: Prevent caching of current page and cookie        
-        self.disableHttpCaching()
 
     def finish(self):
         """ General cleanup on end of request
@@ -1505,6 +1476,10 @@ class RequestTwisted(RequestBase):
             
             RequestBase.__init__(self, properties)
 
+        except MoinMoinNoFooter: # might be triggered by http_redirect
+            self.http_headers() # send headers (important for sending MOIN_ID cookie)
+            self.finish()
+
         except Exception, err:
             # Wrap err inside an internal error if needed
             from MoinMoin import error
@@ -2003,7 +1978,7 @@ class RequestModPy(RequestBase):
         """ Sends out headers and possibly sets default content-type
             and status.
 
-            @keyword more_headers: list of strings, defaults to []
+            @param more_headers: list of strings, defaults to []
         """
         for header in more_headers + getattr(self, 'user_headers', []):
             self.setHttpHeader(header)
