@@ -65,9 +65,9 @@ class BaseExpression:
             return None
     
     def costs(self):
-        """ estimated time to calculate this term
+        """ Return estimated time to calculate this term
         
-        Number is is relative to other terms and has no real unit.
+        Number is relative to other terms and has no real unit.
         It allows to do the fast searches first.
         """ 
         return 0
@@ -81,13 +81,7 @@ class BaseExpression:
 
     def _build_re(self, pattern, use_re=False, case=False):
         """ Make a regular expression out of a text pattern """
-        if case:
-            # case sensitive
-            flags = re.U
-        else:
-            # ignore case
-            flags = re.U | re.I
-            
+        flags = case and re.U or (re.I | re.U)
         if use_re:
             try:
                 self.search_re = re.compile(pattern, flags)
@@ -135,15 +129,13 @@ class AndExpression(BaseExpression):
     def pageFilter(self):
         """ Return a page filtering function
 
-        This function is used to filter page list before we search
-        it.
+        This function is used to filter page list before we search it.
 
-        Return a function that get a page name, and return bool, or None.
+        Return a function that gets a page name, and return bool, or None.
         """
         # Sort terms by cost, then get all title searches
         self.sortByCost()
-        terms = [term for term in self._subterms
-                 if isinstance(term, TitleSearch)]
+        terms = [term for term in self._subterms if isinstance(term, TitleSearch)]
         if terms:
             # Create and return a filter function
             def filter(name):
@@ -227,16 +219,14 @@ class TextSearch(BaseExpression):
         self.negated = 0
         self.use_re = use_re
         self.case = case
-        self._build_re(self._pattern,
-                       use_re=use_re, case=case)
+        self._build_re(self._pattern, use_re=use_re, case=case)
         self.titlesearch = TitleSearch(self._pattern, use_re=use_re, case=case)
         
     def costs(self):
         return 10000
     
     def __unicode__(self):
-        if self.negated: neg = '-'
-        else: neg = ''
+        neg = self.negated and '-' or ''
         return u'%s"%s"' % (neg, unicode(self._pattern))
 
     def highlight_re(self):
@@ -271,9 +261,9 @@ class TextSearch(BaseExpression):
         or_term.add(term, False, False)
         pattern = self._pattern.lower()
         if self.use_re:
-            if pattern[0]=='^':
+            if pattern[0] == '^':
                 pattern = pattern[1:]
-            if pattern[:2]=='\b':
+            if pattern[:2] == '\b':
                 pattern = pattern[2:]
             term = RegularExpressionQuery(Term("text", pattern))
         else:
@@ -281,7 +271,7 @@ class TextSearch(BaseExpression):
             terms = [list(tokenizer(t)) for t in terms]
             term = BooleanQuery()
             for t in terms:
-                if len(t)==1:
+                if len(t) == 1:
                     term.add(CamelCaseQuery(Term("text", t[0])), True, False)
                 else:
                     phrase = PhraseQuery()
@@ -293,7 +283,8 @@ class TextSearch(BaseExpression):
             #term = TermQuery(Term("text", pattern))
         or_term.add(term, False, False)
         return or_term
-        
+
+
 class TitleSearch(BaseExpression):
     """ Term searches in pattern in page title only """
 
@@ -314,8 +305,7 @@ class TitleSearch(BaseExpression):
         return 100
 
     def __unicode__(self):
-        if self.negated: neg = '-'
-        else: neg = ''
+        neg = self.negated and '-' or ''
         return u'%s!"%s"' % (neg, unicode(self._pattern))
 
     def highlight_re(self):
@@ -349,13 +339,15 @@ class TitleSearch(BaseExpression):
     def lupy_term(self):
         pattern = self._pattern.lower()
         if self.use_re:
-            if pattern[0]=='^': pattern = pattern[1:]
+            if pattern[0] == '^':
+                pattern = pattern[1:]
             term = RegularExpressionQuery(Term("title", pattern))
         else:
-            term = PrefixQuery(Term("title", pattern), 3)
-        #term.boost = 100.0
+            term = PrefixQuery(Term("title", pattern), 1000000) # number of chars which are ignored behind the match
+            #term.boost = 100.0
         return term
-    
+
+
 class LinkSearch(BaseExpression):
     """ Search the term in the pagelinks """
 
@@ -379,8 +371,7 @@ class LinkSearch(BaseExpression):
 
     def _build_re(self, pattern, use_re=False, case=False):
         """ Make a regular expression out of a text pattern """
-        flags = (re.U | re.I, re.U)[case]
-
+        flags = case and re.U or (re.I | re.U)
         try:
             if not use_re:
                 raise re.error
@@ -394,7 +385,8 @@ class LinkSearch(BaseExpression):
         return 5000 # cheaper than a TextSearch
 
     def __unicode__(self):
-        return u'%s!"%s"' % (('', '-')[self.negated], unicode(self._pattern))
+        neg = self.negated and '-' or ''
+        return u'%s!"%s"' % (neg, unicode(self._pattern))
 
     def highlight_re(self):
         return u"(%s)" % self._textpattern    
@@ -418,7 +410,7 @@ class LinkSearch(BaseExpression):
             if results:
                 matches.extend(results)
             else: #This happens e.g. for pages that use navigation macros
-                matches.append(TextMatch(0,0))
+                matches.append(TextMatch(0, 0))
 
         # Decide what to do with the results.
         if ((self.negated and matches) or
@@ -433,7 +425,8 @@ class LinkSearch(BaseExpression):
     def lupy_term(self):        
         pattern = self.pattern
         if self.use_re:
-            if pattern[0]=="^": pattern = pattern[1:]
+            if pattern[0] == "^":
+                pattern = pattern[1:]
             term = RegularExpressionQuery(Term("links", pattern))
         else:
             term = TermQuery(Term("links", pattern))
@@ -522,6 +515,7 @@ class FoundPage:
 
     def __init__(self, page_name, matches=None, page=None):
         self.page_name = page_name
+        self.attachment = '' # this is not an attachment
         self.page = page
         if matches is None:
             matches = []
@@ -612,14 +606,29 @@ class FoundPage:
 
 class FoundAttachment(FoundPage):
     """ Represent an attachment in search results """
-    pass
+    
+    def __init__(self, page_name, attachment, matches=None, page=None):
+        self.page_name = page_name
+        self.attachment = attachment
+        self.page = page
+        if matches is None:
+            matches = []
+        self._matches = matches
+
+    def weight(self, unique=1):
+        return 1
+
+    def get_matches(self, unique=1, sort='start', type=Match):
+        return []
+
+    def _unique_matches(self, type=Match):
+        return []
 
 
 ##############################################################################
 ### Parse Query
 ##############################################################################
 
-
 class QueryParser:
     """
     Converts a String into a tree of Query objects
@@ -727,6 +736,7 @@ class QueryParser:
         return (text.startswith('"') and text.endswith('"') or
                 text.startswith("'") and text.endswith("'"))
 
+
 ############################################################################
 ### Search results formatting
 ############################################################################
@@ -735,7 +745,7 @@ class SearchResults:
     """ Manage search results, supply different views
 
     Search results can hold valid search results and format them for
-    many requests, until the wiki content change.
+    many requests, until the wiki content changes.
 
     For example, one might ask for full page list sorted from A to Z,
     and then ask for the same list sorted from Z to A. Or sort results
@@ -800,13 +810,22 @@ class SearchResults:
             list = f.number_list
         else:
             list = f.bullet_list
-        querystr = self.querystring()
-            
+
         # Add pages formatted as list
         if self.hits:
             write(list(1))
 
             for page in self.hits:
+                if page.attachment:
+                    querydict = {
+                        'action': 'AttachFile',
+                        'do': 'get',
+                        'target': page.attachment,
+                    }
+                else:
+                    querydict = None
+                querystr = self.querystring(querydict)
+            
                 matchInfo = ''
                 if info:
                     matchInfo = self.formatInfo(f, page)
@@ -843,7 +862,6 @@ class SearchResults:
         self._reset(request, formatter)
         f = formatter
         write = self.buffer.write
-        querystr = self.querystring()
         
         # Add pages formatted as definition list
         if self.hits:
@@ -853,6 +871,17 @@ class SearchResults:
                 matchInfo = ''
                 if info:
                     matchInfo = self.formatInfo(f, page)
+                if page.attachment:
+                    fmt_context = ""
+                    querydict = {
+                        'action': 'AttachFile',
+                        'do': 'get',
+                        'target': page.attachment,
+                    }
+                else:
+                    fmt_context = self.formatContext(page, context, maxlines)
+                    querydict = None
+                querystr = self.querystring(querydict)
                 item = [
                     f.definition_term(1),
                     f.pagelink(1, page.page_name, querystr=querystr),
@@ -861,7 +890,7 @@ class SearchResults:
                     matchInfo,
                     f.definition_term(0),
                     f.definition_desc(1),
-                    self.formatContext(page, context, maxlines),
+                    fmt_context,
                     f.definition_desc(0),
                     ]
                 write(''.join(item))
@@ -883,7 +912,7 @@ class SearchResults:
         if not page.page:
             page.page = Page(self.request, page.page_name)
         body = page.page.get_raw_body()
-        last = len(body) -1
+        last = len(body) - 1
         lineCount = 0
         output = []
         
@@ -916,7 +945,7 @@ class SearchResults:
                     break
 
             # Add all matches in context and the text between them 
-            while 1:
+            while True:
                 match = matches[j]
                 # Ignore matches behind the current position
                 if start < match.end:
@@ -1038,6 +1067,13 @@ class SearchResults:
         # Add text after match
         if start < len(pagename):
             output.append(f.text(pagename[start:]))
+        
+        if page.attachment: # show the attachment that matched
+            output.extend([
+                    " ",
+                    f.strong(1),
+                    f.text("(%s)" % page.attachment),
+                    f.strong(0)])
 
         return ''.join(output)
 
@@ -1048,7 +1084,7 @@ class SearchResults:
         text. Matches behind location are ignored and an empty string is
         returned.
 
-        @param text: text containing match
+        @param body: text containing match
         @param match: search match in text
         @param location: current location in text
         @rtype: unicode
@@ -1065,11 +1101,12 @@ class SearchResults:
             return ''.join(output)
         return ''
 
-    def querystring(self):
+    def querystring(self, querydict=None):
         """ Return query string, used in the page link """
-        querystr = {'highlight': self.query.highlight_re()}
-        querystr = wikiutil.makeQueryString(querystr)
-        querystr = wikiutil.escape(querystr)
+        if querydict is None:
+            querydict = {'highlight': self.query.highlight_re()}
+        querystr = wikiutil.makeQueryString(querydict)
+        #querystr = wikiutil.escape(querystr)
         return querystr
 
     def formatInfo(self, formatter, page):
@@ -1096,8 +1133,7 @@ class SearchResults:
     def _reset(self, request, formatter):
         """ Update internal state before new output
 
-        Do not calls this, it should be called only by the instance
-        code.
+        Do not call this, it should be called only by the instance code.
 
         Each request might need different translations or other user
         preferences.
@@ -1109,6 +1145,7 @@ class SearchResults:
         _ = request.getText    
         self.matchLabel = (_('match'), _('matches'))
 
+
 ##############################################################################
 ### Searching
 ##############################################################################
@@ -1120,9 +1157,10 @@ class Search:
         self.request = request
         self.query = query
         self.filtered = False
+        self.fs_rootpage = "FS" # XXX FS hardcoded
 
     def run(self):
-        """ Preform search and return results object """
+        """ Perform search and return results object """
         start = time.time()
         if self.request.cfg.lupy_search:
             hits = self._lupySearch()
@@ -1132,11 +1170,17 @@ class Search:
         # important - filter deleted pages or pages the user may not read!
         if not self.filtered:
             hits = self._filter(hits)
+        
+        result_hits = []
+        for page, attachment, match in hits:
+            if attachment:
+                result_hits.append(FoundAttachment(page.page_name, attachment))
+            else:
+                result_hits.append(FoundPage(page.page_name, match))
             
-        hits = [FoundPage(page.page_name, match) for page, match in hits]
         elapsed = time.time() - start
         count = self.request.rootpage.getPageCount()
-        return SearchResults(self.query, hits, count, elapsed)
+        return SearchResults(self.query, result_hits, count, elapsed)
 
     # ----------------------------------------------------------------
     # Private!
@@ -1147,36 +1191,44 @@ class Search:
         Get a list of pages using fast lupy search and return moin
         search in those pages.
         """
+        pages = None
         index = Index(self.request)
-        if not index.exists():
-            return self._moinSearch()
-        self.request.clock.start('_lupySearch')
-        try:
-            hits = index.search(self.query.lupy_term())
-            pages = [hit.get('pagename') for hit in hits]
-        except index.LockedException:
-            pages = None
-        self.request.clock.stop('_lupySearch')
-        if pages == []:
-            return pages
+        if index.exists():
+            self.request.clock.start('_lupySearch')
+            try:
+                hits = index.search(self.query.lupy_term())
+                pages = [(hit.get('pagename'), hit.get('attachment')) for hit in hits]
+            except index.LockedException:
+                pass
+            self.request.clock.stop('_lupySearch')
         return self._moinSearch(pages)
 
     def _moinSearch(self, pages=None):
-        """ Search pages using moin built in full text search 
+        """ Search pages using moin's built-in full text search 
         
         Return list of tuples (page, match). The list may contain
         deleted pages or pages the user may not read.
         """
         self.request.clock.start('_moinSearch')
         from MoinMoin.Page import Page
-        if not pages:
-            pages = self._getPageList()
+        if pages is None:
+            # if we are not called from _lupySearch, we make a full pagelist,
+            # but don't search attachments (thus attachment name = '')
+            pages = [(p, '') for p in self._getPageList()]
         hits = []
-        for name in pages:
-            page = Page(self.request, name)
-            match = self.query.search(page)
-            if match:
-                hits.append((page, match))
+        fs_rootpage = self.fs_rootpage
+        for pagename, attachment in pages:
+            page = Page(self.request, pagename)
+            if attachment:
+                if pagename == fs_rootpage: # not really an attachment
+                    page = Page(self.request, "%s%s" % (fs_rootpage, attachment))
+                    hits.append((page, None, None))
+                else:
+                    hits.append((page, attachment, None))
+            else:
+                match = self.query.search(page)
+                if match:
+                    hits.append((page, attachment, match))
         self.request.clock.stop('_moinSearch')
         return hits
 
@@ -1199,8 +1251,9 @@ class Search:
     def _filter(self, hits):
         """ Filter out deleted or acl protected pages """
         userMayRead = self.request.user.may.read
-        filtered = [(page, match) for page, match in hits
-                    if page.exists() and userMayRead(page.page_name)]    
+        fs_rootpage = self.fs_rootpage + "/"
+        filtered = [(page, attachment, match) for page, attachment, match in hits
+                    if page.exists() and userMayRead(page.page_name) or page.page_name.startswith(fs_rootpage)]    
         return filtered
         
         
