@@ -16,7 +16,7 @@ _debug = 0
 
 from MoinMoin import caching, config, wikiutil
 from MoinMoin.Page import Page
-from MoinMoin.util import MoinMoinNoFooter, datetime
+from MoinMoin.util import MoinMoinNoFooter
 from MoinMoin.formatter.text_html import Formatter
 from MoinMoin.logfile import eventlog, logfile
 
