@@ -16,7 +16,6 @@ from MoinMoin.util import MoinMoinNoFooter
 
 
 def linkto(pagename, request, params=''):
-    from MoinMoin.util import web
     _ = request.getText
 
     if not request.cfg.chart_options:
@@ -31,12 +30,12 @@ def linkto(pagename, request, params=''):
 
     # Create escaped query string from dict and params
     querystr = {'action': 'chart', 'type': 'pagesize'}
-    querystr = web.makeQueryString(querystr)
+    querystr = wikiutil.makeQueryString(querystr)
     querystr = wikiutil.escape(querystr)
     if params:
         querystr += '&amp;' + params
     
-    # TODO: remove escape=0 in 1.4
+    # TODO: remove escape=0 in 2.0
     data = {'url': page.url(request, querystr, escape=0)}
     data.update(request.cfg.chart_options)
     result = ('<img src="%(url)s" width="%(width)d" height="%(height)d"'
