@@ -12,12 +12,6 @@
     cgitb
         from python 2.2 + patches (see XXX)
 
-    optik
-        powerful, flexible, extensible, easy-to-use command-line
-        parsing library for Python, by Greg Ward <gward@python.net>,
-        see http://optik.sourceforge.net/.
-        2003-01-30: imported version 1.4
-
     @copyright: 2001-2004 by Jrgen Hermann <jh@web.de>
     @license: GNU GPL, see COPYING for details.
 """
