@@ -530,7 +530,7 @@ class FCGIbase:
         try:
             if self.__port:
                 s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-                #s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 0)
+                s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
                 # bind to the localhost
                 s.bind(('127.0.0.1', self.__port))
                 s.listen(1)
@@ -592,4 +592,4 @@ class unTHFCGI(FCGIbase):
         while 1:
             (conn, addr) = self.sock.accept()
             self.accept_handler(conn, addr)
-   
\ No newline at end of file
+   
