@@ -134,7 +134,6 @@ class ThemeBase:
         admin inserted in the config file. Everything it enclosed inside
         a div with id="logo".
         
-        @param d: parameter dictionary
         @rtype: unicode
         @return: logo html
         """
@@ -203,7 +202,6 @@ class ThemeBase:
         """
         request = self.request
         _ = request.getText
-        preferencesPage = wikiutil.getSysPage(request, 'UserPreferences')
         
         userlinks = []
         # Add username/homepage link for registered users. We don't care
@@ -215,24 +213,22 @@ class ThemeBase:
             if not aliasname:
                 aliasname = name
             title = "%s @ %s" % (aliasname, interwiki[0])
-            homelink = (request.formatter.interwikilink(1, title=title, *interwiki) +
+            # link to (interwiki) user homepage
+            homelink = (request.formatter.interwikilink(1, title=title, id="userhome", *interwiki) +
                         request.formatter.text(name) +
-                        request.formatter.interwikilink(0))
+                        request.formatter.interwikilink(0, title=title, id="userhome", *interwiki))
             userlinks.append(homelink)        
-            # Set pref page to localized Preferences page
-            title = preferencesPage.split_title(request)
-            userlinks.append(preferencesPage.link_to(request, text=title))
-            
+            # link to userprefs action
+            userlinks.append(d['page'].link_to(request, text=_('Preferences'),
+                                               querystr={'action': 'userprefs'}, id="userprefs"))
+           
         if request.cfg.show_login:
             if request.user.valid:
-                userlinks.append("""\
-<form action="" method="POST">
-<input type="hidden" name="action" value="userform">
-<input type="submit" name="logout" value="%(logout)s">
-</form>
-""" % { 'logout': _('Logout') })
+                userlinks.append(d['page'].link_to(request, text=_('Logout', formatted=False),
+                                                   querystr={'action': 'logout', 'logout': 'logout'}, id="logout"))
             else:
-                userlinks.append(preferencesPage.link_to(request, text=_("Login")))
+                userlinks.append(d['page'].link_to(request, text=_("Login", formatted=False),
+                                                   querystr={'action': 'login'}, id="login"))
 
         userlinks = [u'<li>%s</li>' % link for link in userlinks]
         html = u'<ul id="username">%s</ul>' % ''.join(userlinks)
@@ -301,7 +297,7 @@ class ThemeBase:
                 return (pagename,
                         self.request.formatter.interwikilink(True, interwiki, page) +
                         page +
-                        self.request.formatter.interwikilink(False)
+                        self.request.formatter.interwikilink(False, interwiki, page)
                         )
         
         except ValueError:
@@ -326,7 +322,7 @@ class ThemeBase:
         If you don't like to do this in your theme, or want to use
         different algorithm, override this method.
         
-        @param text: page name, unicode
+        @param name: page name, unicode
         @rtype: unicode
         @return: shortened version.
         """
@@ -471,7 +467,7 @@ class ThemeBase:
         d['title'] = title % d
         d['i18ntitle'] = self.request.getText(d['title'], formatted=False)
         img_src = self.make_icon(icon, d)
-        return wikiutil.link_tag(self.request, page_params % d, img_src, attrs='title="%(i18ntitle)s"' % d)
+        return wikiutil.link_tag(self.request, page_params % d, img_src, title="%(i18ntitle)s" % d)
 
     def msg(self, d):
         """ Assemble the msg display
@@ -520,10 +516,9 @@ class ThemeBase:
                         # links, using _ for spaces.
                         page = page.replace('_', ' ')
                         if request.cfg.interwikiname != interwiki:
-                            link = (self.request.formatter.interwikilink(
-                                True, interwiki, page) +
+                            link = (self.request.formatter.interwikilink(True, interwiki, page) +
                                     self.shortenPagename(page) +
-                                    self.request.formatter.interwikilink(False))
+                                    self.request.formatter.interwikilink(False, interwiki, page))
                             items.append('<li>%s</li>' % link)
                             continue
                         else:
@@ -890,6 +885,7 @@ function actionsMenuInit(title) {
         menu = [
             'raw',
             'print',
+            'RenderAsDocbook',
             'refresh',
             '__separator__',
             'SpellCheck',
@@ -898,6 +894,12 @@ function actionsMenuInit(title) {
             '__separator__',
             'RenamePage',
             'DeletePage',
+            '__separator__',
+            'MyPages',
+            'SubscribeUser',
+            '__separator__',
+            'Despam',
+            'PackagePages',
             ]
 
         titles = {
@@ -913,6 +915,11 @@ function actionsMenuInit(title) {
             'DeletePage': _('Delete Page', formatted=False),
             'LikePages': _('Like Pages', formatted=False),
             'LocalSiteMap': _('Local Site Map', formatted=False),
+            'MyPages': _('My Pages', formatted=False),
+            'SubscribeUser': _('Subscribe User', formatted=False),
+            'Despam': _('Remove Spam', formatted=False),
+            'PackagePages': _('Package Pages', formatted=False),
+            'RenderAsDocbook': _('Render as Docbook', formatted=False),
             }
 
         options = []
@@ -945,8 +952,9 @@ function actionsMenuInit(title) {
 
             options.append(option % data)
 
-        # Add custom actions not in the standard menu
-        more = [item for item in available if not item in titles]
+        # Add custom actions not in the standard menu, except for
+        # some actions like AttachFile (we have them on top level)
+        more = [item for item in available if not item in titles and not item in ('AttachFile',)]
         more.sort()
         if more:
             # Add separator
@@ -1080,18 +1088,18 @@ actionsMenuInit('%(label)s');
         if self.showBothEditLinks() and guiworks:
             text = _('Edit (Text)', formatted=False)
             params = params + 'text'
-            attrs = 'name="texteditlink"'
+            attrs = {'name': "texteditlink"}
         else:
             text = _('Edit', formatted=False)
             if guiworks:
                 # 'textonly' will be upgraded dynamically to 'guipossible' by JS
                 params = params + 'textonly'
-                attrs = 'name="editlink"'
+                attrs = {'name': "editlink"}
             else:
                 params = params + 'text'
-                attrs = 'name="texteditlink"'
+                attrs = {'name': "texteditlink"}
         
-        return wikiutil.link_tag(self.request, params, text, attrs=attrs)
+        return wikiutil.link_tag(self.request, params, text, **attrs)
 
     def showBothEditLinks(self):
         """ Return True if both edit links should be displayed """
