@@ -8,7 +8,7 @@
 
 import string, time, re
 from MoinMoin import user, util, wikiutil
-from MoinMoin.util import web, mail, datetime
+from MoinMoin.util import web, mail, timefuncs
 from MoinMoin.widget import html
 
 _debug = 0
@@ -64,6 +64,9 @@ class UserSettingsHandler:
         _ = self._
         form = self.request.form
     
+        if form.has_key('cancel'):
+            return
+
         if form.has_key('account_sendmail'):
             if not self.cfg.mail_enabled:
                 return _("""This wiki is not enabled for mail processing.
@@ -89,7 +92,7 @@ Contact the owner of the wiki, who can enable email.""")
                 return _("Use UserPreferences to change your settings or create an account.")
             # Create user profile
             if form.has_key('create'):
-                theuser = self.request.get_user()
+                theuser = self.request.get_user_from_form()
             else:
                 theuser = user.User(self.request, auth_method="request:152")
                 
@@ -138,7 +141,7 @@ space between words. Group page name is not allowed.""") % wikiutil.escape(theus
                 return _("Please provide your email address. If you lose your"
                          " login information, you can get it by email.")
 
-            # Email should be unique - see also MoinMoin/scripts/moin_usercheck.py
+            # Email should be unique - see also MoinMoin/script/accounts/moin_usercheck.py
             if theuser.email and self.request.cfg.user_email_unique:
                 users = user.getUserList(self.request)
                 for uid in users:
@@ -157,18 +160,21 @@ space between words. Group page name is not allowed.""") % wikiutil.escape(theus
             if _debug:
                 result = result + util.dumpFormData(form)
             return result
-            
-        if form.has_key('select_user'): # Select user profile (su user)
+        
+        
+        # Select user profile (su user) - only works with cookie auth active.
+        if form.has_key('select_user'):
             if (wikiutil.checkTicket(self.request.form['ticket'][0]) and
-                self.request.user.name in self.request.cfg.superuser and
-                self.request.request_method == 'POST'):
+                self.request.request_method == 'POST' and
+                self.request.user.isSuperUser()):
                 su_user = form.get('selected_user', [''])[0]
                 uid = user.getUserId(self.request, su_user)
                 theuser = user.User(self.request, uid)
                 theuser.disabled = None
                 theuser.save()
+                from MoinMoin import auth
+                auth.setCookie(self.request, theuser)
                 self.request.user = theuser
-                self.request.setCookie()
                 return  _("Use UserPreferences to change settings of the selected user account")
             else:
                 return _("Use UserPreferences to change your settings or create an account.")
@@ -176,7 +182,7 @@ space between words. Group page name is not allowed.""") % wikiutil.escape(theus
         if form.has_key('save'): # Save user profile
             if self.request.request_method != 'POST':
                 return _("Use UserPreferences to change your settings or create an account.")
-            theuser = self.request.get_user()
+            theuser = self.request.get_user_from_form()
                 
             if not 'name' in theuser.auth_attribs:
                 # Require non-empty name
@@ -227,7 +233,7 @@ space between words. Group page name is not allowed.""") % wikiutil.escape(theus
                 return _("Please provide your email address. If you lose your"
                          " login information, you can get it by email.")
 
-            # Email should be unique - see also MoinMoin/scripts/moin_usercheck.py
+            # Email should be unique - see also MoinMoin/script/accounts/moin_usercheck.py
             if theuser.email and self.request.cfg.user_email_unique:
                 users = user.getUserList(self.request)
                 for uid in users:
@@ -361,7 +367,7 @@ class UserSettings:
             options.append((
                 str(offset),
                 '%s [%s%s:%s]' % (
-                    time.strftime(self.cfg.datetime_fmt, util.datetime.tmtuple(t)),
+                    time.strftime(self.cfg.datetime_fmt, timefuncs.tmtuple(t)),
                     "+-"[offset < 0],
                     string.zfill("%d" % (abs(offset) / 3600), 2),
                     string.zfill("%d" % (abs(offset) % 3600 / 60), 2),
@@ -471,7 +477,7 @@ class UserSettings:
         _ = self._
         self.make_form()
 
-        if self.request.user.name in self.request.cfg.superuser:
+        if self.request.user.isSuperUser():
             ticket = wikiutil.createTicket()
             self.make_row(_('Select User'), [self._user_select()])
             self._form.append(html.INPUT(type="hidden", name="ticket", value="%s" % ticket))
@@ -485,7 +491,7 @@ class UserSettings:
             self.make_row('', button_cell)
             
         if self.request.user.valid and not create_only:
-            buttons = [('save', _('Save'))]
+            buttons = [('save', _('Save')), ('cancel', _('Cancel')), ]
             uf_remove = self.cfg.user_form_remove
             uf_disable = self.cfg.user_form_disable
             for attr in self.request.user.auth_attribs:
@@ -520,7 +526,7 @@ class UserSettings:
                     self._tz_select(),
                     html.BR(),
                     _('Server time is'), ' ',
-                    time.strftime(self.cfg.datetime_fmt, util.datetime.tmtuple()),
+                    time.strftime(self.cfg.datetime_fmt, timefuncs.tmtuple()),
                     ' (UTC)',
                 ])
 
@@ -578,8 +584,9 @@ class UserSettings:
             buttons = [
                 # IMPORTANT: login should be first to be the default
                 # button when a user hits ENTER.
-                ('login', _('Login')),
-                ("create", _('Create Profile')),
+                #('login', _('Login')),  # we now have a Login macro
+                ('create', _('Create Profile')),
+                ('cancel', _('Cancel')),
             ]
             for key, label, type, length, textafter in self.cfg.user_form_fields:
                 if key in ('name', 'password', 'password2', 'email'):
@@ -615,6 +622,71 @@ def getUserForm(request, create_only=False):
     return UserSettings(request).asHTML(create_only=create_only)
 
 
+class Login:
+    """ User login. """
+
+    def __init__(self, request):
+        """ Initialize user settings form.
+        """
+        self.request = request
+        self._ = request.getText
+        self.cfg = request.cfg
+
+    def make_row(self, label, cell, **kw):
+        """ Create a row in the form table.
+        """
+        self._table.append(html.TR().extend([
+            html.TD(**kw).extend([html.B().append(label), '   ']),
+            html.TD().extend(cell),
+        ]))
+
+
+    def asHTML(self):
+        """ Create the complete HTML form code. """
+        _ = self._
+        request = self.request
+        sn = request.getScriptname()
+        pi = request.getPathinfo()
+        action = u"%s%s" % (sn, pi)
+        userprefslink = wikiutil.getSysPage(request, "UserPreferences").link_to(request)
+        hint = _("To create an account or recover a lost password, see the %(userprefslink)s page.") % {
+               'userprefslink': userprefslink}
+        self._form = html.FORM(action=action)
+        self._table = html.TABLE(border="0")
+
+        # Use the user interface language and direction
+        lang_attr = request.theme.ui_lang_attr()
+        self._form.append(html.Raw('<div class="userprefs"%s>' % lang_attr))
+
+        self._form.append(html.INPUT(type="hidden", name="action", value="login"))
+        self._form.append(self._table)
+        self._form.append(html.P().append(hint))
+        self._form.append(html.Raw("</div>"))
+
+        self.make_row(_('Name'), [
+            html.INPUT(
+                type="text", size="32", name="name",
+            ),
+        ])
+
+        self.make_row(_('Password'), [
+            html.INPUT(
+                type="password", size="32", name="password",
+            ),
+        ])
+
+        self.make_row('', [
+            html.INPUT(
+                type="submit", name='login', value=_('Login')
+            ),
+        ])
+
+        return unicode(self._form)
+
+def getLogin(request):
+    """ Return HTML code for the login. """
+    return Login(request).asHTML()
+
 #############################################################################
 ### User account administration
 #############################################################################
