@@ -567,23 +567,30 @@ def do_edit(pagename, request):
     if savetext is None:
         pg.sendEditor()
         return
+  
+    # did user hit cancel button?
+    cancelled = request.form.has_key('button_cancel')
 
     # convert input from Graphical editor
-    if lasteditor == 'gui':
-        from MoinMoin.converter.text_html_text_x_moin import convert
-        savetext = convert(request, pagename, savetext) # XXX error handling
+    from MoinMoin.converter.text_html_text_x_moin import convert, ConvertError
+    try:
+        if lasteditor == 'gui':
+            savetext = convert(request, pagename, savetext)
+                
+        # IMPORTANT: normalize text from the form. This should be done in
+        # one place before we manipulate the text.
+        savetext = pg.normalizeText(savetext, stripspaces=rstrip)
+    except ConvertError:
+        # we don't want to throw an exception if user cancelled anyway
+        if not cancelled:
+            raise
 
-    # IMPORTANT: normalize text from the form. This should be done in
-    # one place before we manipulate the text.
-    savetext = pg.normalizeText(savetext, stripspaces=rstrip)
-
-    comment = wikiutil.clean_comment(comment)
-
-    # Edit was canceled
-    if request.form.has_key('button_cancel'):
+    if cancelled:
         pg.sendCancel(savetext or "", rev)
         return
 
+    comment = wikiutil.clean_comment(comment)
+
     # Add category
 
     # TODO: this code does not work with extended links, and is doing
@@ -775,8 +782,6 @@ def do_raw(pagename, request):
         Page(request, pagename).send_page(request)
         return
 
-    request.http_headers(["Content-type: text/plain;charset=%s" % config.charset])
-
     if request.form.has_key('rev'):
         try:
             rev = request.form['rev'][0]
@@ -790,10 +795,7 @@ def do_raw(pagename, request):
     else:
         page = Page(request, pagename)
 
-    text = page.get_raw_body()
-    text = page.encodeTextMimeType(text)
-    request.write(text)
-    raise MoinMoinNoFooter
+    page.send_raw()
 
 
 def do_format(pagename, request):
