@@ -7,9 +7,10 @@
 """
     
 import os, re, difflib, urllib, cgi
+import codecs
 
 from MoinMoin import util, version, config
-from MoinMoin.util import pysupport
+from MoinMoin.util import pysupport, filesys
 
 # Exceptions
 class InvalidFileNameError(Exception):
@@ -37,7 +38,6 @@ def decodeWindowsPath(text):
     """
 
     import locale
-    import codecs
     cur_charset = locale.getdefaultlocale()[1]
     try:
         return unicode(text, 'utf-8')
@@ -82,7 +82,7 @@ def decodeUserInput(s, charsets=[config.charset]):
     Decodes input from the user.
     
     @param s: the string to unquote
-    @param charset: the charset to assume the string is in
+    @param charsets: list of charsets to assume the string is in
     @rtype: unicode
     @return: the unquoted string as unicode
     """
@@ -202,9 +202,9 @@ def quoteWikinameURL(pagename, charset=config.charset):
     Use urllib.quote to quote any character that is not always safe. 
 
     @param pagename: the original pagename (unicode)
-    @charset: url text encoding, 'utf-8' recommended. Other charsert
-              might not be able to encode the page name and raise
-              UnicodeError. (default config.charset ('utf-8')).
+    @param charset: url text encoding, 'utf-8' recommended. Other charsert
+                    might not be able to encode the page name and raise
+                    UnicodeError. (default config.charset ('utf-8')).
     @rtype: string
     @return: the quoted filename, all unsafe characters encoded
     """
@@ -331,7 +331,7 @@ def unquoteWikiname(filename, charsets=[config.charset]):
     Todo: search clients of unquoteWikiname and check for exceptions. 
 
     @param filename: string using charset and possibly quoted parts
-    @param charset: charset used by string
+    @param charsets: list of charsets used by string
     @rtype: Unicode String
     @return: WikiName
     """
@@ -393,7 +393,91 @@ def version2timestamp(v):
         This must ONLY be used for display purposes.
     """
     return v/1000000.0
+
+
+# This is the list of meta attribute names to be treated as integers.
+# IMPORTANT: do not use any meta attribute names with "-" (or any other chars
+# invalid in python attribute names), use e.g. _ instead.
+INTEGER_METAS = ['current', 'revision', # for page storage (moin 2.0)
+                 'data_format_revision', # for data_dir format spec (use by mig scripts)
+                ]
+
+class MetaDict(dict):
+    """ store meta informations as a dict """
+    def __init__(self, metafilename):
+        """ create a MetaDict from metafilename """
+        dict.__init__(self)
+        self.metafilename = metafilename
+        self.dirty = False
+        self.loaded = False
+
+    def _get_meta(self):
+        """ get the meta dict from an arbitrary filename.
+            does not keep state, does uncached, direct disk access.
+            @param metafilename: the name of the file to read
+            @return: dict with all values or {} if empty or error
+        """
+        # XXX what does happen if the metafile is being written to in another process?
+        try:
+            metafile = codecs.open(self.metafilename, "r", "utf-8")
+            meta = metafile.read() # this is much faster than the file's line-by-line iterator
+            metafile.close()
+        except IOError:
+            meta = u''
+        for line in meta.splitlines():
+            key, value = line.split(':', 1)
+            value = value.strip()
+            if key in INTEGER_METAS:
+                value = int(value)
+            dict.__setitem__(self, key, value)
+        self.loaded = True
     
+    def _put_meta(self):
+        """ put the meta dict into an arbitrary filename.
+            does not keep or modify state, does uncached, direct disk access.
+            @param metafilename: the name of the file to write
+            @param metadata: dict of the data to write to the file
+        """
+        meta = []
+        for key, value in self.items():
+            if key in INTEGER_METAS:
+                value = str(value)
+            meta.append("%s: %s" % (key, value))
+        meta = '\n'.join(meta)
+        # XXX what does happen if the metafile is being read or written to in another process?
+        metafile = codecs.open(self.metafilename, "w", "utf-8")
+        metafile.write(meta)
+        metafile.close()
+        filesys.chmod(self.metafilename, 0666 & config.umask)
+        self.dirty = False
+
+    def sync(self, mtime_usecs=None):
+        """ sync the in-memory dict to disk (if dirty) """
+        if self.dirty:
+            if not mtime_usecs is None:
+                self.__setitem__('mtime', str(mtime_usecs))
+            self._put_meta()
+
+    def __getitem__(self, key):
+        try:
+            return dict.__getitem__(self, key)
+        except KeyError:
+            if not self.loaded:
+                self._get_meta() # lazy loading of metadata
+                return dict.__getitem__(self, key)
+            else:
+                raise
+
+    def __setitem__(self, key, value):
+        try:
+            oldvalue = dict.__getitem__(self, key)
+        except KeyError:
+            oldvalue = None
+        if value != oldvalue:
+            dict.__setitem__(self, key, value)
+            self.dirty = True
+
+
 #############################################################################
 ### InterWiki
 #############################################################################
@@ -413,10 +497,8 @@ def split_wiki(wikiurl):
         try:
             wikitag, tail = wikiurl.split("/", 1)
         except ValueError:
-            wikitag = None
-            tail = None
-
-    return (wikitag, tail)
+            wikitag, tail = 'Self', wikiurl
+    return wikitag, tail
 
 
 def join_wiki(wikiurl, wikitail):
@@ -488,7 +570,7 @@ def resolve_wiki(request, wikiurl):
     wikitag, tail = split_wiki(wikiurl)
 
     # return resolved url
-    if wikitag and _interwiki_list.has_key(wikitag):
+    if _interwiki_list.has_key(wikitag):
         return (wikitag, _interwiki_list[wikitag], tail, False)
     else:
         return (wikitag, request.getScriptname(), "/InterWiki", True)
@@ -721,7 +803,7 @@ def importPlugin(cfg, kind, name, function="execute"):
         return importBuiltinPlugin(kind, name, function)
 
 
-def importWikiPlugin(cfg, kind, name, function):
+def importWikiPlugin(cfg, kind, name, function="execute"):
     """ Import plugin from the wiki data directory
     
     See importPlugin docstring.
@@ -732,7 +814,7 @@ def importWikiPlugin(cfg, kind, name, function):
     return importNameFromPlugin(moduleName, function)
 
 
-def importBuiltinPlugin(kind, name, function):
+def importBuiltinPlugin(kind, name, function="execute"):
     """ Import builtin plugin from MoinMoin package 
     
     See importPlugin docstring.
@@ -846,8 +928,8 @@ def parseAttributes(request, attrstring, endtoken=None, extension=None):
     Parse a list of attributes and return a dict plus a possible
     error message.
     If extension is passed, it has to be a callable that returns
-    None when it was not interested into the token, '' when all was OK
-    and it did eat the token, and any other string to return an error
+    a tuple (found_flag, msg). found_flag is whether it did find and process
+    something, msg is '' when all was OK or any other string to return an error
     message.
     
     @param request: the request object
@@ -878,9 +960,13 @@ def parseAttributes(request, attrstring, endtoken=None, extension=None):
 
         # call extension function with the current token, the parser, and the dict
         if extension:
-            msg = extension(key, parser, attrs)
-            if msg == '': continue
-            if msg: break
+            found_flag, msg = extension(key, parser, attrs)
+            #request.log("%r = extension(%r, parser, %r)" % (msg, key, attrs))
+            if found_flag:
+                continue
+            elif msg:
+                break
+            #else (we found nothing, but also didn't have an error msg) we just continue below:
 
         try:
             eq = parser.get_token()
@@ -990,17 +1076,25 @@ def isPicture(url):
 def link_tag(request, params, text=None, formatter=None, on=None, **kw):
     """ Create a link.
 
+    TODO: cleanup css_class
+
     @param request: the request object
     @param params: parameter string appended to the URL after the scriptname/
     @param text: text / inner part of the <a>...</a> link - does NOT get
                  escaped, so you can give HTML here and it will be used verbatim
     @param formatter: the formatter object to use
-    @keyword on: opening/closing tag only
-    @keyword attrs: additional attrs (HTMLified string)
+    @param on: opening/closing tag only
+    @keyword attrs: additional attrs (HTMLified string) (removed in 1.5.3)
     @rtype: string
     @return: formatted link tag
     """
-    css_class = kw.get('css_class', None)
+    if kw.has_key('css_class'):
+        css_class = kw['css_class']
+        del kw['css_class'] # one time is enough
+    else:
+        css_class = None
+    id = kw.get('id', None)
+    name = kw.get('name', None)
     if text is None:
         text = params # default
     if formatter:
@@ -1014,10 +1108,12 @@ def link_tag(request, params, text=None, formatter=None, on=None, **kw):
         return '</a>'
     
     attrs = ''
-    if kw.has_key('attrs'):
-        attrs += ' ' + kw['attrs']
     if css_class:
         attrs += ' class="%s"' % css_class
+    if id:
+        attrs += ' id="%s"' % id
+    if name:
+        attrs += ' name="%s"' % name
     result = '<a%s href="%s/%s">' % (attrs, request.getScriptname(), params)
     if on:
         return result
