@@ -21,12 +21,19 @@ sys.path.insert(0, '/path/to/wikiconfig')
 # Path of the directory where farmconfig is located (if different).
 ## sys.path.insert(0, '/path/to/farmconfig')
 
+# Debug mode - show detailed error reports
+## import os
+## os.environ['MOIN_DEBUG'] = '1'
 
 from MoinMoin.server.standalone import StandaloneConfig, run
 
 
 class Config(StandaloneConfig):
 
+    # Server name
+    # Used to create .log, .pid and .prof files
+    name = 'moin'
+
     # Path to moin shared files (default '/usr/share/moin/wiki/htdocs')
     # If you installed with --prefix=PREFIX, use 'PREFIX/share/moin/wiki/htdocs'
     docs = '/usr/share/moin/htdocs'
@@ -46,7 +53,7 @@ class Config(StandaloneConfig):
 
     # Log (default commented)
     # Log is written to stderr or to a file you specify here.
-    ## logPath = 'moin.log'
+    ## logPath = name + '.log'
 
     # Server class (default ThreadPoolServer)
     # 'ThreadPoolServer' - create a constant pool of threads, simplified
@@ -69,21 +76,24 @@ class Config(StandaloneConfig):
     # Request queue size (default 50)
     # The size of the socket listen backlog.
     requestQueueSize = 50
+
+    # Properties
+    # Allow overriding any request property by the value defined in
+    # this dict e.g properties = {'script_name': '/mywiki'}.
+    properties = {}
     
     # Memory profile (default commented)
     # Useful only if you are a developer or interested in moin memory usage
-    # A memory profile named 'moin_standalone--2004-09-27--01-24.log' is
+    # A memory profile named 'moin--2004-09-27--01-24.log' is
     # created each time you start the server.
     ## from MoinMoin.util.profile import Profiler
-    ## memoryProfile = Profiler('moin_standalone',
-    ##                          requestsPerSample=100,
-    ##                          collect=0)
+    ## memoryProfile = Profiler(name, requestsPerSample=100, collect=0)
 
     # Hotshot profile (default commented)
-    ## import hotshot
-    ## hotshotProfile = hotshot.Profile("moin.prof")
+    # Not compatible with threads - use with SimpleServer only.
+    ## hotshotProfile = name + '.prof'
 
 
-# Run moin moin server:
-run(Config)
+if __name__ == '__main__':
+    run(Config)
 
